"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
# Standard library imports
import hashlib
import logging
import os
import re
import shutil
import tempfile
import uuid
import zipfile
from collections import defaultdict
from datetime import datetime
from pathlib import Path
import socket
from typing import List

# Related third party imports
import pytz
import simplejson as json

# Local application imports
from faraday_plugins.plugins.plugins_utils import its_cve, its_cwe

logger = logging.getLogger("faraday").getChild(__name__)

VALID_SERVICE_STATUS = ("open", "closed", "filtered")
VULN_SKIP_FIELDS_TO_HASH = ['run_date']


class PluginBase:
    # TODO: Add class generic identifier
    class_signature = "PluginBase"

    def __init__(self, *args, **kwargs):
        # Must be unique. Check that there is not
        # an existent plugin with the same id.
        # TODO: Make script that list current ids.
        self.ignore_info = kwargs.get("ignore_info", False)
        self.hostname_resolution = kwargs.get("hostname_resolution", True)
        self.vuln_tag = kwargs.get("vuln_tag", None)
        self.host_tag = kwargs.get("host_tag", None)
        self.service_tag = kwargs.get("service_tag", None)
        self.default_vuln_tag = None
        self.id = None
        self.auto_load = True
        self._rid = id(self)
        self.version = None
        self.name = None
        self.description = ""
        self._command_regex = None
        self.command = None
        self._output_file_path = None
        self._use_temp_file = False
        self._delete_temp_file = False
        self._temp_file_extension = "tmp"
        self._current_path = None
        self.framework_version = None
        self._completition = {}
        self._new_elems = []
        self._settings = {}
        self.command_id = None
        self._cache = {}
        self._hosts_cache = {}
        self._service_cache = {}
        self._vulns_cache = {}
        self.start_date = datetime.utcnow()
        self.logger = logger.getChild(self.__class__.__name__)
        self.open_options = {"mode": "r", "encoding": "utf-8"}
        self.plugin_version = "0.0"
        self.vulns_data = {"hosts": [], "command": {"tool": "",
                                                    "command": "",
                                                    "params": "",
                                                    "user": "",
                                                    "hostname": "",
                                                    "start_date": self.start_date.isoformat(),
                                                    "duration": 0,
                                                    "import_source": "report"}}

    def __str__(self):
        return f"Plugin: {self.id}"

    def _get_temp_file(self, extension="tmp"):
        temp_dir = tempfile.gettempdir()
        temp_filename = f"{self.id}_{next(tempfile._get_candidate_names())}.{extension}"
        temp_file_path = os.path.join(temp_dir, temp_filename)
        return temp_file_path

    def resolve_hostname(self, hostname):
        if not self.hostname_resolution:
            return hostname
        if not hostname:
            self.logger.error(f"Hostname provided is None or Empty {hostname}, using 0.0.0.0 as ip")
            return "0.0.0.0"
        try:
            socket.inet_aton(hostname)  # is already an ip
            return hostname
        except OSError:
            pass
        try:
            ip_address = socket.gethostbyname(hostname)
        except Exception as e:
            return hostname
        else:
            return ip_address

    @staticmethod
    def get_utctimestamp(date):
        if date is not None:
            try:
                utc_date = date.astimezone(pytz.UTC)
                return utc_date.timestamp()
            except Exception as e:
                logger.error(f"Error generating timestamp: {e}")
                return None
        else:
            return date

    @staticmethod
    def normalize_severity(severity):
        if severity is not None:
            severity = str(severity).lower()
        else:
            severity = ""

        def align_string_based_vulns(severity):
            severities = ['info', 'low', 'med', 'high', 'critical']
            for sev in severities:
                if severity[0:3] in sev:
                    return sev
            return severity

        severity = align_string_based_vulns(severity)
        # Transform numeric severity into desc severity
        numeric_severities = {"0": "info",
                              "1": "low",
                              "2": "med",
                              "3": "high",
                              "4": "critical"}
        if severity not in numeric_severities.values():
            severity = numeric_severities.get(severity, 'unclassified')
        return severity

    # Caches
    def get_from_cache(self, cache_id):
        return self._cache.get(cache_id, None)

    def save_host_cache(self, host):
        cache_id = self.get_host_cache_id(host)
        if cache_id not in self._hosts_cache:
            obj_uuid = self.save_cache(host)
            self.vulns_data["hosts"].append(host)
            self._hosts_cache[cache_id] = obj_uuid
        else:
            obj_uuid = self._hosts_cache[cache_id]
            if host['hostnames']:
                chached_host = self.get_from_cache(obj_uuid)
                chached_host['hostnames'] = list(set(chached_host['hostnames'] + host['hostnames']))
        return obj_uuid

    def save_service_cache(self, host_id, service):
        cache_id = self.get_host_service_cache_id(host_id, service)
        if cache_id not in self._service_cache:
            obj_uuid = self.save_cache(service)
            host = self.get_from_cache(host_id)
            host["services"].append(service)
            self._service_cache[cache_id] = obj_uuid
        else:
            obj_uuid = self._service_cache[cache_id]
        return obj_uuid

    def save_service_vuln_cache(self, host_id, service_id, vuln):
        if self.ignore_info and vuln['severity'] == 'info':
            return None
        else:
            cache_id = self.get_service_vuln_cache_id(host_id, service_id, vuln)
            if cache_id not in self._vulns_cache:
                obj_uuid = self.save_cache(vuln)
                service = self.get_from_cache(service_id)
                service["vulnerabilities"].append(vuln)
                self._vulns_cache[cache_id] = obj_uuid
            else:
                obj_uuid = self._vulns_cache[cache_id]
            return obj_uuid

    def save_host_vuln_cache(self, host_id, vuln):
        if self.ignore_info and vuln['severity'] == 'info':
            return None
        else:
            cache_id = self.get_host_vuln_cache_id(host_id, vuln)
            if cache_id not in self._vulns_cache:
                obj_uuid = self.save_cache(vuln)
                host = self.get_from_cache(host_id)
                host["vulnerabilities"].append(vuln)
                self._vulns_cache[cache_id] = obj_uuid
            else:
                obj_uuid = self._vulns_cache[cache_id]
            return obj_uuid

    @staticmethod
    def _get_dict_hash(d, keys):
        return hash(frozenset(map(lambda x: (x, d.get(x, None)), keys)))

    @classmethod
    def get_host_cache_id(cls, host):
        cache_id = cls._get_dict_hash(host, ['ip'])
        return cache_id

    @classmethod
    def get_host_service_cache_id(cls, host_id, service):
        service_copy = service.copy()
        service_copy.update({"host_cache_id": host_id})
        cache_id = cls._get_dict_hash(service_copy, ['host_cache_id', 'protocol', 'port'])
        return cache_id

    @classmethod
    def get_service_vuln_cache_id(cls, host_id, service_id, vuln):
        vuln_copy = vuln.copy()
        vuln_copy.update({"host_cache_id": host_id, "service_cache_id": service_id})
        cache_id = cls._get_dict_hash(vuln_copy,
                                      ['host_cache_id', 'service_cache_id', 'name', 'desc', 'website', 'path', 'pname',
                                       'method'])
        return cache_id

    @classmethod
    def get_host_vuln_cache_id(cls, host_id, vuln):
        vuln_copy = vuln.copy()
        vuln_copy.update({"host_cache_id": host_id})
        cache_id = cls._get_dict_hash(vuln_copy,
                                      ['host_cache_id', 'name', 'desc', 'website', 'path', 'pname', 'method'])
        return cache_id

    def save_cache(self, obj):
        obj_uuid = uuid.uuid1()
        self._cache[obj_uuid] = obj
        return obj_uuid

    def report_belongs_to(self, **kwargs):
        return False

    def has_custom_output(self):
        return bool(self._output_file_path)

    def get_custom_file_path(self):
        return self._output_file_path

    def set_actions_queue(self, _pending_actions):
        """
            We use plugin controller queue to add actions created by plugins.
            Plugin controller will consume this actions.

        :param controller: plugin controller
        :return: None
        """
        self._pending_actions = _pending_actions

    def setCommandID(self, command_id):
        self.command_id = command_id

    def getSettings(self):
        for param, (param_type, value) in self._settings.items():
            yield param, value

    def get_ws(self):  # TODO Borrar
        return ""

    def getSetting(self, name):
        setting_type, value = self._settings[name]
        return value

    def addSetting(self, param, param_type, value):
        self._settings[param] = param_type, value

    def updateSettings(self, new_settings):
        for name, value in new_settings.items():
            if name in self._settings:
                setting_type, curr_value = self._settings[name]
                self._settings[name] = setting_type, setting_type(value)

    def canParseCommandString(self, current_input):
        """
        This method can be overridden in the plugin implementation
        if a different kind of check is needed
        """
        if (self._command_regex is not None and
                self._command_regex.match(current_input.strip()) is not None):
            self.command = self.get_command(current_input)
            return True

    def get_command(self, current_input: str) -> str:
        command = self._command_regex.findall(current_input)[0]
        if isinstance(command, tuple):
            return "".join(command).strip()

        return command.strip()

    def processCommandString(self, username, current_path, command_string):
        """
        With this method a plugin can add additional arguments to the
        command that it's going to be executed.
        """
        self._current_path = current_path
        if command_string.startswith(("sudo", "python", "python3")):
            params = " ".join(command_string.split()[2:])
        else:
            params = " ".join(command_string.split()[1:])
        self.vulns_data["command"]["params"] = params if not self.ignore_info else f"{params} (Info ignored)"
        self.vulns_data["command"]["user"] = username
        self.vulns_data["command"]["import_source"] = "shell"
        if self._use_temp_file:
            self._delete_temp_file = True
            self._output_file_path = self._get_temp_file(extension=self._temp_file_extension)
        return None

    def getCompletitionSuggestionsList(self, current_input):
        """
        This method can be overridden in the plugin implementation
        if a different kind of check is needed
        """
        words = current_input.split(" ")
        cword = words[len(words) - 1]
        options = {}
        for k, v in self._completition.items():
            if re.search(str("^" + cword), k, flags=re.IGNORECASE):
                options[k] = v
        return options

    def processOutput(self, command_output):
        if self.has_custom_output():
            self._parse_filename(Path(self.get_custom_file_path()))
        else:
            self.parseOutputString(command_output)

    def _parse_filename(self, filename: Path):
        with filename.open(**self.open_options) as output:
            self.parseOutputString(output.read())
        if self._delete_temp_file:
            try:
                if filename.is_file():
                    os.remove(filename)
                elif filename.is_dir():
                    shutil.rmtree(filename)
            except Exception as e:
                self.logger.error(f"Error on delete file: ({filename}) [{e}]")

    def processReport(self, filepath: Path, user="faraday"):
        if type(filepath) == str:  # TODO workaround for compatibility, remove in the future
            filepath = Path(filepath)
        if filepath.is_file():
            self.vulns_data["command"]["params"] = filepath.name if not self.ignore_info else f"{filepath.name} (Info ignored)"
            self.vulns_data["command"]["user"] = user
            self.vulns_data["command"]["import_source"] = "report"
            self._parse_filename(filepath)
        else:
            raise FileNotFoundError(filepath)

    def parseOutputString(self, output):
        """
        This method must be implemented.
        This method will be called when the command finished executing and
        the complete output will be received to work with it
        Using the output the plugin can create and add hosts, interfaces,
        services, etc.
        """
        raise NotImplementedError('This method must be implemented.')

    def createAndAddHost(self, name, os="unknown", hostnames=None, mac=None, description="", tags=None):

        if not hostnames:
            hostnames = []
        if not isinstance(hostnames, list):
            hostnames = [hostnames]
        # Some plugins sends a list with None, we filter empty and None values.
        hostnames = [hostname for hostname in hostnames if hostname]
        if os is None:
            os = "unknown"
        if tags is None:
            tags = []
        if isinstance(tags, str):
            tags = [tags]
        if self.host_tag:
            if isinstance(self.host_tag, list):
                tags += self.host_tag
            else:
                tags.append(self.host_tag)
        host = {"ip": name, "os": os, "hostnames": hostnames, "description": description, "mac": mac,
                "credentials": [], "services": [], "vulnerabilities": [], "tags": tags}
        host_id = self.save_host_cache(host)
        return host_id

    def createAndAddServiceToHost(self, host_id, name,
                                  protocol="tcp", ports=None,
                                  status="open", version="",
                                  description="", tags=None):
        if ports:
            if isinstance(ports, list):
                ports = int(ports[0])
            elif isinstance(ports, str):
                ports = int(ports)
        if not protocol:
            protocol = "tcp"
        if status not in VALID_SERVICE_STATUS:
            status = 'open'
        if tags is None:
            tags = []
        if isinstance(tags, str):
            tags = [tags]
        if self.service_tag:
            if isinstance(self.service_tag, list):
                tags += self.service_tag
            else:
                tags.append(self.service_tag)
        service = {"name": name, "protocol": protocol, "port": ports, "status": status,
                   "version": version, "description": description, "credentials": [], "vulnerabilities": [],
                   "tags": tags}

        service_id = self.save_service_cache(host_id, service)

        return service_id

    @staticmethod
    def modify_refs_struct(ref: List[str]) -> List[dict]:
        """
        Change reference struct from list of strings to a list of dicts with the form of {name, type}
        """
        if not ref:
            return []
        refs = []
        for r in ref:
            if isinstance(r, dict):
                refs.append(r)
            else:
                if r.strip():
                    refs.append({'name': r.strip(), 'type': 'other'})
        return refs

    def createAndAddVulnToHost(self, host_id, name, desc="", ref=None,
                               severity="", resolution="", data="", external_id=None, run_date=None,
                               impact=None, custom_fields=None, status="", policyviolations=None,
                               easeofresolution=None, confirmed=False, tags=None, cve=None, cwe=None, cvss2=None,
                               cvss3=None):

        ref = self.modify_refs_struct(ref)
        if status == "":
            status = "open"
        if impact is None:
            impact = {}
        if policyviolations is None:
            policyviolations = []
        if custom_fields is None:
            custom_fields = {}
        if tags is None:
            tags = []
        if isinstance(tags, str):
            tags = [tags]
        if self.vuln_tag:
            if isinstance(self.vuln_tag, list):
                tags += self.vuln_tag
            else:
                tags.append(self.vuln_tag)
        if self.default_vuln_tag:
            if isinstance(self.default_vuln_tag, list):
                tags += self.default_vuln_tag
            else:
                tags.append(self.default_vuln_tag)
        if cve is None:
            cve = []
        elif type(cve) is str:
            cve = [cve]
        cve = its_cve(cve)
        if cwe is None:
            cwe = []
        elif type(cwe) is str:
            cwe = [cwe]
        cwe = its_cwe(cwe)
        if cvss2 is None:
            cvss2 = {}
        if cvss3 is None:
            cvss3 = {}
        vulnerability = {"name": name, "desc": desc, "severity": self.normalize_severity(severity), "refs": ref,
                         "external_id": external_id, "type": "Vulnerability", "resolution": resolution, "data": data,
                         "custom_fields": custom_fields, "status": status, "impact": impact,
                         "policyviolations": policyviolations, "cve":  cve, "cvss3": cvss3, "cvss2": cvss2,
                         "confirmed": confirmed, "easeofresolution": easeofresolution, "tags": tags, "cwe": cwe
                         }
        if run_date:
            vulnerability["run_date"] = self.get_utctimestamp(run_date)
        vulnerability_id = self.save_host_vuln_cache(host_id, vulnerability)
        return vulnerability_id

    def createAndAddVulnToService(self, host_id, service_id, name, desc="",
                                  ref=None, severity="", resolution="", data="", external_id=None, run_date=None,
                                  custom_fields=None, policyviolations=None, impact=None, status="",
                                  confirmed=False, easeofresolution=None, tags=None, cve=None, cwe=None, cvss2=None,
                                  cvss3=None):
        ref = self.modify_refs_struct(ref)
        if status == "":
            status = "open"
        if impact is None:
            impact = {}
        if policyviolations is None:
            policyviolations = []
        if custom_fields is None:
            custom_fields = {}
        if tags is None:
            tags = []
        if isinstance(tags, str):
            tags = [tags]
        if self.vuln_tag:
            if isinstance(self.vuln_tag, list):
                tags += self.vuln_tag
            else:
                tags.append(self.vuln_tag)
        if self.default_vuln_tag:
            if isinstance(self.default_vuln_tag, list):
                tags += self.default_vuln_tag
            else:
                tags.append(self.default_vuln_tag)
        if cve is None:
            cve = []
        elif type(cve) is str:
            cve = [cve]
        cve = its_cve(cve)
        if cwe is None:
            cwe = []
        elif type(cwe) is str:
            cwe = [cwe]
        cwe = its_cwe(cwe)
        if cvss2 is None:
            cvss2 = {}
        if cvss3 is None:
            cvss3 = {}
        vulnerability = {"name": name, "desc": desc, "severity": self.normalize_severity(severity), "refs": ref,
                         "external_id": external_id, "type": "Vulnerability", "resolution": resolution, "data": data,
                         "custom_fields": custom_fields, "status": status, "impact": impact,
                         "policyviolations": policyviolations, "cve": cve, "cvss3": cvss3, "cvss2": cvss2,
                         "easeofresolution": easeofresolution, "confirmed": confirmed, "tags": tags, "cwe": cwe
                         }
        if run_date:
            vulnerability["run_date"] = self.get_utctimestamp(run_date)
        vulnerability_id = self.save_service_vuln_cache(host_id, service_id, vulnerability)
        return vulnerability_id

    def createAndAddVulnWebToService(self, host_id, service_id, name, desc="",
                                     ref=None, severity="", resolution="",
                                     website="", path="", request="",
                                     response="", method="", pname="",
                                     params="", query="", category="", data="", external_id=None,
                                     confirmed=False, status="", easeofresolution=None, impact=None,
                                     policyviolations=None, status_code=None, custom_fields=None, run_date=None,
                                     tags=None, cve=None, cvss2=None, cvss3=None, cwe=None):
        if params is None:
            params = ""
        if method is None:
            method = ""
        if pname is None:
            pname = ""
        if query is None:
            query = ""
        if website is None:
            website = ""
        if path is None:
            path = ""
        if request is None:
            request = ""
        if response is None:
            response = ""
        ref = self.modify_refs_struct(ref)
        if status == "":
            status = "open"
        if impact is None:
            impact = {}
        if policyviolations is None:
            policyviolations = []
        if custom_fields is None:
            custom_fields = {}
        if tags is None:
            tags = []
        if isinstance(tags, str):
            tags = [tags]
        if self.vuln_tag:
            if isinstance(self.vuln_tag, list):
                tags += self.vuln_tag
            else:
                tags.append(self.vuln_tag)
        if self.default_vuln_tag:
            if isinstance(self.default_vuln_tag, list):
                tags += self.default_vuln_tag
            else:
                tags.append(self.default_vuln_tag)
        if cve is None:
            cve = []
        elif type(cve) is str:
            cve = [cve]
        cve = its_cve(cve)
        if cwe is None:
            cwe = []
        elif type(cwe) is str:
            cwe = [cwe]
        cwe = its_cwe(cwe)
        if cvss2 is None:
            cvss2 = {}
        if cvss3 is None:
            cvss3 = {}
        vulnerability = {"name": name, "desc": desc, "severity": self.normalize_severity(severity), "refs": ref,
                         "external_id": external_id, "type": "VulnerabilityWeb", "resolution": resolution,
                         "data": data, "website": website, "path": path, "request": request, "response": response,
                         "method": method, "pname": pname, "params": params, "query": query, "category": category,
                         "confirmed": confirmed, "status": status, "easeofresolution": easeofresolution,
                         "impact": impact, "policyviolations": policyviolations, "cve": cve,  "cvss3": cvss3,
                         "cvss2": cvss2, "status_code": status_code, "custom_fields": custom_fields, "tags": tags,
                         "cwe": cwe}
        if run_date:
            vulnerability["run_date"] = self.get_utctimestamp(run_date)
        vulnerability_id = self.save_service_vuln_cache(host_id, service_id, vulnerability)
        return vulnerability_id

    def createAndAddNoteToHost(self, host_id, name, text):
        return None

    def createAndAddNoteToService(self, host_id, service_id, name, text):
        return None

    def createAndAddNoteToNote(self, host_id, service_id, note_id, name, text):
        return None

    def createAndAddCredToService(self, host_id, service_id, username, password):
        credential = {"name": "credential", "username": username, "password": password}
        service = self.get_from_cache(service_id)
        service["credentials"].append(credential)
        credential_id = self.save_cache(credential)
        return credential_id

    def get_data(self):
        self.vulns_data["command"]["tool"] = self.id
        self.vulns_data["command"]["command"] = self.command if self.command else self.id
        self.vulns_data["command"]["duration"] = (datetime.utcnow() - self.start_date).microseconds
        return self.vulns_data

    def get_json(self):
        self.logger.debug("Generate Json")
        return json.dumps(self.get_data())

    def get_summary(self):
        plugin_json = self.get_data()
        summary = {'hosts': len(plugin_json['hosts']), 'services': 0,
                   'hosts_vulns': sum(list(map(lambda x: len(x['vulnerabilities']), plugin_json['hosts']))),
                   'services_vulns': 0, 'severity_vulns': defaultdict(int),
                   'vuln_hashes': []
                   }
        hosts_with_services = filter(lambda x: len(x['services']) > 0, plugin_json['hosts'])
        host_services = list(map(lambda x: x['services'], hosts_with_services))
        summary['services'] = sum(map(lambda x: len(x), host_services))
        services_vulns = 0
        for host in plugin_json['hosts']:
            for vuln in host['vulnerabilities']:
                summary['severity_vulns'][vuln['severity']] += 1
        for services in host_services:
            for service in services:
                services_vulns += len(service['vulnerabilities'])
                for vuln in service['vulnerabilities']:
                    summary['severity_vulns'][vuln['severity']] += 1
        summary['services_vulns'] = services_vulns
        for obj_uuid in self._vulns_cache.values():
            vuln = self.get_from_cache(obj_uuid)
            vuln_copy = vuln.copy()
            for field in VULN_SKIP_FIELDS_TO_HASH:
                vuln_copy.pop(field, None)
            dict_hash = hashlib.sha1(json.dumps(vuln_copy).encode()).hexdigest() # nosec
            summary['vuln_hashes'].append(dict_hash)
        return summary


# TODO Borrar
class PluginTerminalOutput(PluginBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def processOutput(self, term_output):
        try:
            self.parseOutputString(term_output)
        except Exception as e:
            self.logger.error(e)


# TODO Borrar
class PluginCustomOutput(PluginBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def processOutput(self, term_output):
        # we discard the term_output since it's not necessary
        # for this type of plugins
        self.processReport(Path(self._output_file_path))


class PluginByExtension(PluginBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.extension = []

    def report_belongs_to(self, extension="", **kwargs):
        match = False
        if type(self.extension) == str:
            match = (self.extension == extension)
        elif type(self.extension) == list:
            match = (extension in self.extension)
        self.logger.debug(f"Extension Match: [{extension} =/in {self.extension}] -> {match}")
        return match


class PluginXMLFormat(PluginByExtension):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identifier_tag = []
        self.identifier_tag_attributes = {}
        self.extension = ".xml"
        self.open_options = {"mode": "rb"}

    def report_belongs_to(self, main_tag="", main_tag_attributes={}, **kwargs):
        match = False
        if super().report_belongs_to(**kwargs):
            if type(self.identifier_tag) == str:
                match = (main_tag == self.identifier_tag)
            elif type(self.identifier_tag) == list:
                match = (main_tag in self.identifier_tag)
            if self.identifier_tag_attributes:
                match = self.identifier_tag_attributes.issubset(main_tag_attributes)
            self.logger.debug(f"Tag Match: [{main_tag} =/in {self.identifier_tag}] -> {match}")
        return match


class PluginJsonFormat(PluginByExtension):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.json_keys = set()
        self.filter_keys = set()
        self.extension = ".json"

    def report_belongs_to(self, file_json_keys=None, **kwargs):
        match = False
        if super().report_belongs_to(**kwargs):
            if file_json_keys is None:
                file_json_keys = set()
            if self.filter_keys & file_json_keys:
                return match
            if isinstance(self.json_keys, list):
                for jk in self.json_keys:
                    match = jk.issubset(file_json_keys)
                    self.logger.debug(f"Json Keys Match: [{file_json_keys} =/in {jk}] -> {match}")
                    if match:
                        break
            else:
                match = self.json_keys.issubset(file_json_keys)
                self.logger.debug(f"Json Keys Match: [{file_json_keys} =/in {self.json_keys}] -> {match}")
        return match


class PluginMultiLineJsonFormat(PluginByExtension):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.json_keys = set()
        self.extension = ".json"

    def report_belongs_to(self, file_json_keys=None, **kwargs):
        match = False
        report_path = kwargs.get("report_path", "")
        if super().report_belongs_to(**kwargs):
            with open(report_path) as f:
                try:
                    json_lines = list(map(lambda x: json.loads(x), f.readlines()))
                    if len(json_lines) > 0:
                        matched_lines = list(filter(lambda json_line: self.json_keys.issubset(json_line.keys()),
                                                    json_lines))
                        match = len(matched_lines) == len(json_lines)
                        self.logger.debug(f"Json Keys Match: [{json_lines[0].keys()} =/in {self.json_keys}] -> {match}")
                except ValueError:
                    return False
        return match


class PluginCSVFormat(PluginByExtension):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.extension = ".csv"
        self.csv_headers = set()

    def report_belongs_to(self, file_csv_headers=None, **kwargs):
        match = False
        if file_csv_headers is None:
            file_csv_headers = set()
        if super().report_belongs_to(**kwargs):
            if isinstance(self.csv_headers, list):
                match = bool(list(filter(lambda x: x.issubset(file_csv_headers), self.csv_headers)))
            else:
                match = self.csv_headers.issubset(file_csv_headers)
            self.logger.debug(f"CSV Headers Match: [{file_csv_headers} =/in {self.csv_headers}] -> {match}")
        return match


class PluginZipFormat(PluginByExtension):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.extension = ".zip"
        self.files_list = set()

    def _parse_filename(self, filename):
        file = zipfile.ZipFile(filename, "r")
        self.parseOutputString(file)

    def report_belongs_to(self, files_in_zip=None, **kwargs):
        match = False
        if super().report_belongs_to(**kwargs):
            if files_in_zip is None:
                files_in_zip = set()
            match = bool(self.files_list & files_in_zip)
            self.logger.debug(f"Files List Match: [{files_in_zip} =/in {self.files_list}] -> {match}")
        return match



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

import xml.etree.ElementTree as ET

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "1.0"
__maintainer__ = "Blas Moyano"
__status__ = "Development"


class NcrackParser:
    def __init__(self, xml_output):
        self.tree = self.parse_xml(xml_output)
        if self.tree:
            scanner = self.tree.attrib.get('scanner', None)
            args = self.tree.attrib.get('args', None)
            start = self.tree.attrib.get('start', None)
            start_str = self.tree.attrib.get('start_str', None)
            service_data = None if self.tree.find('service') is None else \
                self.get_service(self.tree.findall('service'))
            self.ncrack_info = {
                "scanner_name": scanner,
                "args": args,
                "date": start,
                "date_str": start_str,
                "info_service": service_data
            }
        else:
            self.tree = None

    def parse_xml(self, xml_output):
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f'SyntaxError In xml: {err}. {xml_output}')
            return None
        return tree

    def get_service(self, tree):
        list_service_info = []
        for service in tree:
            address = service.find('address')
            port = service.find('port')
            credential = service.find('credentials')
            if address is not None:
                addr = address.attrib.get('addr', None)
                addr_type = address.attrib.get('addrtype', None)
            else:
                addr = None
                addr_type = None

            if port is not None:
                protocol = port.attrib.get('protocol', None)
                port_number = port.attrib.get('portid', None)
                port_name = port.attrib.get('name', None)
            else:
                protocol = None
                port_number = None
                port_name = None

            if credential is not None:
                user = credential.attrib.get('username', None)
                passw = credential.attrib.get('password', None)
            else:
                user = None
                passw = None

            service_info = {
                "addr": addr,
                "addr_type": addr_type,
                "protocol": protocol,
                "port_number": port_number,
                "port_name": port_name,
                "user": user,
                "passw": passw
            }
            list_service_info.append(service_info)
        return list_service_info


class NcrackPlugin(PluginXMLFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "ncrackrun"
        self.id = 'ncrack'
        self.name = 'ncrack XML Plugin'
        self.plugin_version = '0.0.1'
        self.version = '1.0.0'
        self.framework_version = '1.0.0'

    def parseOutputString(self, output):
        parser = NcrackParser(output)
        data = parser.ncrack_info

        for service_vuln in data['info_service']:
            host_id = self.createAndAddHost(service_vuln['addr'],
                                            description=f"{data['scanner_name']} - args: {data['args']}")

            service_id = self.createAndAddServiceToHost(host_id,
                                                        service_vuln['addr'],
                                                        ports=service_vuln['port_number'],
                                                        protocol=service_vuln['protocol'],
                                                        description=service_vuln['port_name'])
            if service_vuln['user'] is not None or service_vuln['passw'] is not None:
                self.createAndAddCredToService(host_id,
                                               service_id,
                                               username=service_vuln['user'],
                                               password=service_vuln['passw'])


def createPlugin(*args, **kwargs):
    return NcrackPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import xml.etree.ElementTree as ET

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.plugins_utils import CVE_regex

__author__ = "Micaela Ranea Sanchez"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato", "Federico Kirschbaum",
               "Micaela Ranea Sanchez", "German Riera"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Micaela Ranea Sanchez"
__email__ = "mranea@infobytesec.com"
__status__ = "Development"


class NexposeFullXmlParser:
    """
    The objective of this class is to parse Nexpose's XML 2.0 Report.

    TODO: Handle errors.
    TODO: Test nexpose output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param xml_filepath A proper xml generated by nexpose
    """

    def __init__(self, xml_output):
        tree = self.parse_xml(xml_output)
        self.vulns = self.get_vuln_definitions(tree)

        if tree:
            self.items = self.get_items(tree, self.vulns)
        else:
            self.items = []

    @staticmethod
    def get_severity_from_report(score):
        try:
            if type(score) != float:
                score = float(score)

            cvss_ranges = [(0.0, 3.4, 'med'),
                           (3.5, 7.4, 'high'),
                           (7.5, 10.1, 'critical')]
            for (lower, upper, severity) in cvss_ranges:
                if lower <= score < upper:
                    return severity
        except ValueError:
            return 'unclassified'

    @staticmethod
    def parse_xml(xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            return None

        return tree

    def parse_html_type(self, node: ET.Element) -> str:
        """
        Parse XML element of type HtmlType

        @return ret A string containing the parsed element
        """
        ret = ""
        tag: str = node.tag.lower()
        if tag == 'containerblockelement':
            if len(list(node)) > 0:
                for child in list(node):
                    ret += self.parse_html_type(child)
            else:
                ret += node.text.strip() if node.text else ""
        if tag == 'listitem':
            if len(list(node)) > 0:
                for child in list(node):
                    ret += self.parse_html_type(child)
            else:
                ret = node.text.strip() if node.text else ""
        if tag == 'orderedlist':
            i = 1
            for item in list(node):
                ret += "\t" + str(i) + " " + self.parse_html_type(item) + "\n"
                i += 1
        if tag == 'paragraph':
            if len(list(node)) > 0:
                for child in list(node):
                    ret += self.parse_html_type(child)
            else:
                ret += node.text.strip() if node.text else ""
        if tag == 'unorderedlist':
            for item in list(node):
                ret += "\t" + "* " + self.parse_html_type(item) + "\n"
        if tag == 'urllink':
            if node.get('text'):
                ret += node.text.strip() + " "
            last = ""
            for attr in node.attrib:
                if node.get(attr) and node.get(attr) != node.get(last):
                    ret += node.get(attr) + " "
                last = attr

        return ret

    def parse_tests_type(self, node, vulnsDefinitions):
        """
        Parse XML element of type TestsType

        @return vulns A list of vulnerabilities according to vulnsDefinitions
        """
        vulns = list()

        for tests in node.findall('tests'):
            for test in tests.iter('test'):
                if test.get('id').lower() in vulnsDefinitions:
                    vuln = vulnsDefinitions[test.get('id').lower()].copy()
                    key = test.get('key', '')
                    vuln['pci'] = test.get('pci-compliance-status')
                    vuln['vulnerable_since'] = test.get('vulnerable-since')
                    vuln['scan_id'] = test.get('scan-id')
                    if key.startswith('/'):
                        # It has the path where the vuln was found
                        # Example key: "/comments.asp||content"
                        vuln['path'] = key[:key.find('|')]
                    for desc in list(test):
                        vuln['desc'] += self.parse_html_type(desc)
                    vulns.append(vuln)
        return vulns

    def get_vuln_definitions(self, tree):
        """
        @returns vulns A dict of Vulnerability Definitions
        """
        vulns = dict()

        for vulnsDef in tree.iter('VulnerabilityDefinitions'):
            for vulnDef in vulnsDef.iter('vulnerability'):
                vid = vulnDef.get('id').lower()
                vector = vulnDef.get('cvssVector')
                vuln = {
                    'desc': "",
                    'name': vulnDef.get('title'),
                    'refs': [],
                    'resolution': "",
                    'severity': self.get_severity_from_report(vulnDef.get('severity')),
                    'tags': list(),
                    'is_web': vid.startswith('http-'),
                    'risk': vulnDef.get('riskScore'),
                    'CVE': [],
                    'cvss2': {
                        "vector_string": vector.replace("(", "").replace(")", "") if vector else None
                    }
                }

                for item in list(vulnDef):
                    if item.tag == 'description':
                        for htmlType in list(item):
                            vuln['desc'] += self.parse_html_type(htmlType)
                    if item.tag == 'exploits':
                        for exploit in list(item):
                            if exploit.get('title') and exploit.get('link') and exploit.get('type') \
                                    and exploit.get('skillLevel'):
                                title = exploit.get('title').strip()
                                link = exploit.get('link').strip()
                                type = exploit.get('type').strip()
                                skillLevel = exploit.get('skillLevel').strip()
                                vuln['refs'].append(" ".join([title, link, type, skillLevel]))
                    if item.tag == 'malware':
                        for names in item.findall("name"):
                            nameMalware = names.text
                            vuln['refs'].append(nameMalware)
                    if item.tag == 'references':
                        for ref in list(item):
                            if not ref.text:
                                continue
                            source = ""
                            if "source" in ref.attrib:
                                source = ref.attrib['source'] + ": "
                            rf = ref.text.strip()
                            check = CVE_regex.search(rf.upper())
                            if check:
                                vuln["CVE"].append(check.group())
                            else:
                                if rf.isnumeric():
                                    rf = source + rf
                                vuln['refs'].append(rf)
                    if item.tag == 'solution':
                        for htmlType in list(item):
                            vuln['resolution'] += self.parse_html_type(htmlType)

                """
                # there is currently no method to register tags in vulns
                if item.tag == 'tags':
                    for tag in list(item):
                        vuln['tags'].append(tag.text.lower())
                """
                vulns[vid] = vuln
        return vulns

    def get_items(self, tree, vulns):
        """
        @return hosts A list of Host instances
        """

        hosts = list()

        for nodes in tree.iter('nodes'):
            for node in nodes.iter('node'):
                host = dict()
                host['name'] = node.get('address')
                host['mac'] = node.get('hardware-address')
                host['hostnames'] = list()
                host['os'] = ""
                host['services'] = list()
                host['vulns'] = self.parse_tests_type(node, vulns)

                for names in node.iter('names'):
                    for name in list(names):
                        host['hostnames'].append(name.text)

                for fingerprints in node.iter('fingerprints'):
                    for os_data in fingerprints.iter('os'):
                        os_name = os_data.get('product')
                        if os_name:
                            host['os'] = os_name
                for endpoints in node.iter('endpoints'):
                    for endpoint in list(endpoints):
                        svc = {
                            'protocol': endpoint.get('protocol'),
                            'port': endpoint.get('port'),
                            'status': endpoint.get('status'),
                        }
                        for services in endpoint.iter('services'):
                            for service in list(services):
                                svc['name'] = service.get('name')
                                svc['vulns'] = self.parse_tests_type(
                                    service, vulns)
                                for configs in service.iter('configurations'):
                                    for config in list(configs):
                                        if "banner" in config.get('name'):
                                            svc['version'] = config.get('name')
                        host['services'].append(svc)
                hosts.append(host)

        return hosts


class NexposeFullPlugin(PluginXMLFormat):
    """
    Example plugin to parse nexpose output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "NexposeReport"
        self.id = "NexposeFull"
        self.name = "Nexpose XML 2.0 Report Plugin"
        self.plugin_version = "0.0.1"
        self.version = "Nexpose Enterprise 5.7.19"
        self.framework_version = "1.0.0"
        self.options = None

    def parseOutputString(self, output):

        parser = NexposeFullXmlParser(output)

        for item in parser.items:
            pattern = '([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$'
            if not item['mac']:
                item['mac'] = '0000000000000000'
            match = re.search(pattern, item['mac'])
            if match:
                mac = item['mac']
            else:
                mac = ':'.join(item['mac'][i:i + 2] for i in range(0, 12, 2))
            h_id = self.createAndAddHost(item['name'], item['os'], hostnames=item['hostnames'], mac=mac)
            for v in item['vulns']:
                v['data'] = {"vulnerable_since": v['vulnerable_since'], "scan_id": v['scan_id'], "PCI": v['pci']}
                self.createAndAddVulnToHost(
                    h_id,
                    v['name'],
                    v['desc'],
                    v['refs'],
                    v['severity'],
                    v['resolution'],
                    cve=v.get('CVE'),
                    cvss2=v.get('cvss2')
                )

            for s in item['services']:
                version = s.get("version", "")
                s_id = self.createAndAddServiceToHost(
                    h_id,
                    s['name'],
                    s['protocol'],
                    ports=[str(s['port'])],
                    status=s['status'],
                    version=version)

                for v in s['vulns']:

                    if v['is_web']:
                        self.createAndAddVulnWebToService(
                            h_id,
                            s_id,
                            v['name'],
                            v['desc'],
                            v['refs'],
                            v['severity'],
                            v['resolution'],
                            cve=v.get('CVE'),
                            path=v.get('path', ''),
                            cvss2=v.get('cvss2')
                        )
                    else:
                        self.createAndAddVulnToService(
                            h_id,
                            s_id,
                            v['name'],
                            v['desc'],
                            v['refs'],
                            v['severity'],
                            v['resolution'],
                            cve=v.get('CVE'),
                            cvss2=v.get('cvss2')
                        )

        del parser


def createPlugin(*args, **kwargs):
    return NexposeFullPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from faraday_plugins.plugins.plugin import PluginBase
import re



__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class ReverseraiderParser:
    """
    The objective of this class is to parse an xml file generated by the reverseraider tool.

    @param reverseraider_filepath A proper simple report generated by reverseraider
    """

    def __init__(self, output):

        lists = output.split("\r\n")
        self.items = []

        if re.search("ReverseRaider domain scanner|Error opening", output) is not None:
            return

        for line in lists:
            if line != "":
                print(f"({line})")
                info = line.split("\t")
                if info.__len__() > 0:
                    item = {'host': info[0], 'ip': info[1]}
                    print(f"host = {info[0]}, ip = {info[1]}")
                    self.items.append(item)


class ReverseraiderPlugin(PluginBase):
    """
    Example plugin to parse reverseraider output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Reverseraider"
        self.name = "Reverseraider XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "0.7.6"
        self.options = None
        self._command_regex = re.compile(r'^(sudo \.\/reverseraider|\.\/reverseraider)\s+.*?')
        self._completition = {
            "": "reverseraider -d domain | -r range [options]",
            "-r": "range of ipv4 or ipv6 addresses, for reverse scanning",
            "-d": "domain, for wordlist scanning (example google.com)",
            "-w": "wordlist file (see wordlists directory...)",
            "-t": "requests timeout in seconds",
            "-P": "enable numeric permutation on wordlist (default off)",
            "-D": "nameserver to use (default: resolv.conf)",
            "-T": "use TCP queries instead of UDP queries",
            "-R": "don't set the recursion bit on queries",
        }


    def parseOutputString(self, output):
        parser = ReverseraiderParser(output)
        for item in parser.items:
            h_id = self.createAndAddHost(item['ip'])
        del parser



def createPlugin(*args, **kwargs):
    return ReverseraiderPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import json
from faraday_plugins.plugins.plugin import PluginJsonFormat

__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Blas Moyano"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"


class SslLabsJsonParser:

    def __init__(self, json_output):
        list_data = json.loads(json_output)
        self.json_data = list_data[0]

    def host_info(self, data):
        host_information = {
            "url": data.get('host', 'Not Info'),
            "description_host": "SSL Labs",
            "port": data.get('port', 0),
            "protocol": data.get('protocol', 'Not Info'),
            "status": data.get('status', 'Not Info'),
            "version": data.get('engineVersion', '0.0.0'),
            "run_date": data.get('startTime', 'Not Info')
        }
        return host_information

    def get_ip(self, data):
        for ip in data:
            return ip.get('ipAddress', '0.0.0.0')

    def get_vulns(self, data):
        chain = data.get('chain', None)
        vuln_list = []
        policies_list = [data.get('hstsPolicy', 'No Information'),
                         data.get('hpkpPolicy', 'No Information'),
                         data.get('hpkpRoPolicy', 'No Information')]

        for vulns in chain['certs']:
            vuln = {
                "name": vulns.get('issuerLabel', 'No Information'),
                "desc": vulns.get('issuerSubject', 'No Information'),
                "data": f'SHA1HASH: {vulns.get("sha1Hash", "No Information")}'
                        f'PINSHA256: {vulns.get("pinSha256", "No Information")}'
                        f'RAW: {vulns.get("raw", "No Information")}',
                "policy": policies_list
            }
            vuln_list.append(vuln)
        return vuln_list


class SslLabsPlugin(PluginJsonFormat):
    """ Handle the SSL Labs tool. Detects the output of the tool
    and adds the information to Faraday.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "ssllabs"
        self.name = "SSL Labs"
        self.plugin_version = "0.1"
        self.version = "3.4.5"
        self.json_keys = {'engineVersion', 'criteriaVersion', 'endpoints'}


    def parseOutputString(self, output):
        parser = SslLabsJsonParser(output)
        host = parser.host_info(parser.json_data)

        host_id = self.createAndAddHost(
            name=parser.get_ip(parser.json_data['endpoints']),
            hostnames=[host['url']],
            description=host['description_host'])

        service_id = self.createAndAddServiceToHost(
            host_id=host_id,
            name=host['url'],
            protocol=host['protocol'],
            ports=host['port'],
            status=host['status'],
            version=host['version'])

        vulns = parser.get_vulns(parser.json_data['endpoints'][0]['details'])

        for vuln in vulns:
            policy_info = f"Long max age: {vuln['policy'][0]['LONG_MAX_AGE']}" \
                          f"Status: {vuln['policy'][0]['status']} | {vuln['policy'][1]['status']} | {vuln['policy'][2]['status']} " \
                          f"directives: {vuln['policy'][0]['directives']} | {vuln['policy'][1]['directives']} | {vuln['policy'][2]['directives']}" \
                          f"pins: {vuln['policy'][1]['directives']} | {vuln['policy'][2]['directives']} " \
                          f"matchedPins: {vuln['policy'][1]['matchedPins']} | {vuln['policy'][2]['matchedPins']} "

            self.createAndAddVulnToService(host_id,
                                           service_id=service_id,
                                           name=vuln['name'],
                                           desc=vuln['desc'],
                                           policyviolations=[policy_info],
                                           data=vuln['data'])


def createPlugin(*args, **kwargs):
    return SslLabsPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from faraday_plugins.plugins.plugin import PluginBase
import re

__author__ = "Andres Tarantini"
__copyright__ = "Copyright (c) 2015 Andres Tarantini"
__credits__ = ["Andres Tarantini"]
__license__ = "MIT"
__version__ = "0.0.1"
__maintainer__ = "Andres Tarantini"
__email__ = "atarantini@gmail.com"
__status__ = "Development"


class SSHDefaultScanPlugin(PluginBase):
    """
    Handle sshdefaultscan (https://github.com/atarantini/sshdefaultscan) output
    using --batch and --batch-template; supports --username and --password
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "sshdefaultscan"
        self.name = "sshdefaultscan"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self._command_regex = re.compile(
            r'^(python sshdefaultscan.py|\./sshdefaultscan.py)\s+.*?')
        self._completition = {"--fast": "Fast scan mode"}

    def parseOutputString(self, output):
        for line in [line.strip() for line in output.split("\n")]:
            output_rexeg_match = re.match(
                r".*:.*@\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$", line)
            if output_rexeg_match:
                credentials, address = line.split("@")
                host = self.createAndAddHost(address)
                service = self.createAndAddServiceToHost(host, "ssh", protocol="tcp", ports=[22])
                username, password = credentials.split(":")
                cred = self.createAndAddCredToService(
                    host, service, username, password)
                vuln = self.createAndAddVulnToService(
                    host,
                    service,
                    "Default credentials",
                    desc="The SSH server have default credentials ({username}:{password})".format(
                        username=username,
                        password=password
                    ),
                    severity=3
                )


    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        if "--batch" not in command_string:
            return "{command} --batch --batch-template {template}".format(
                command=command_string,
                template="{username}:{password}@{host}"
            )
        else:
            return None


def createPlugin(*args, **kwargs):
    return SSHDefaultScanPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2016  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import os
import re
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = 'Ezequiel Tavella'
__copyright__ = 'Copyright 2016, Faraday Project'
__credits__ = ['Ezequiel Tavella', 'Matías Ariel Ré Medina', 'Conrad Stein K']
__license__ = ''
__version__ = '1.0.2'
__status__ = 'Development'


class ArachniXmlParser:
    def __init__(self, xml_output):

        self.tree = self.parse_xml(xml_output)
        if self.tree:
            self.issues = self.getIssues(self.tree)
            self.plugins = self.getPlugins(self.tree)
            self.system = self.getSystem(self.tree)
        else:
            self.system = None
            self.issues = None
            self.plugins = None

    def parse_xml(self, xml_output):
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            return None
        return tree

    def getIssues(self, tree):
        # Get vulnerabilities.
        issues_tree = tree.find('issues')
        for self.issue_node in issues_tree:
            yield Issue(self.issue_node)

    def getPlugins(self, tree):
        # Get info about plugins executed in scan.
        plugins_tree = tree.find('plugins')
        return Plugins(plugins_tree)

    def getSystem(self, tree):
        system_tree = tree.find('system')
        if system_tree is None:
            return System(tree, False)
        else:
            return System(system_tree, True)


class Issue:

    def __init__(self, issue_node):
        self.node = issue_node
        self.name = self.getDesc('name')
        self.severity = self.getDesc('severity')
        self.cwe = self.getDesc('cwe')
        self.remedy_guidance = self.getDesc('remedy_guidance')
        self.description = self.getDesc('description')
        self.var = self.getChildTag('vector', 'affected_input_name')
        self.url = self.getChildTag('vector', 'url')
        self.method = self.getChildTag('vector', 'method')
        self.references = self.getReferences()
        self.parameters = self.getParameters()
        self.request = self.getRequest()
        self.response = self.getResponse()
        self.data = self.getData()

    def getData(self):
        name = self.node.findtext('check/name', '')
        description = self.node.findtext('check/description', '')
        author = self.node.findtext('check/description', '')
        data = ""
        if name:
            data += f'\nname: {name}'
        if description:
            data += f'\ndescription: {description}'
        if author:
            data += f'\nauthor: {author}'

        return data

    def getDesc(self, tag):

        # Get value of tag xml
        description = self.node.find(tag)

        if description is not None and description.text is not None:
            return description.text
        else:
            return 'None'

    def getChildTag(self, main_tag, child_tag):

        # Get value of tag child xml
        main_entity = self.node.find(main_tag)

        if not main_entity:
            return 'None'

        result = main_entity.find(child_tag)

        if result is not None and result.text is not None:
            return result.text
        else:
            return 'None'

    def getReferences(self):
        """
        Returns current issue references on this format
        {'url': 'http://www.site.com', 'name': 'WebSite'}.
        """
        result = []
        references = self.node.find('references')

        if not references:
            return result

        for tag in references.findall('reference'):
            url = tag.get('url')
            result.append(url)

        return result

    def getParameters(self):

        # Get parameters of query
        result = []

        try:
            parameters = self.node.find('vector').find('inputs')
            for param in parameters.findall('input'):
                name = param.get('name')
                result.append(name)
        except:
            parameters = ''

        return ' - '.join(result)

    def getRequest(self):

        # Get data about request.
        try:
            request = self.node.find('page/request')
            raw_data = request.find('raw')
            data = raw_data.text
            if not data:
                data = self.contruct_request(request)
            return data
        except:
            return 'None'

    @staticmethod
    def contruct_request(request):
        data = request.findtext("method", "").upper()
        data += f" {request.findtext('url', '')}"
        for h in request.findall('headers/header'):
            data += f"\n{h.get('name')}: {h.get('value')}"
        if request.findtext('body',""):
            data += f"\n{request.findtext('body','')}"
        return data

    @staticmethod
    def construct_response(request):
        data = ""
        if request.findtext("code", ""):
            data += f'\ncode: {request.findtext("code", "")}'
        if request.findtext("ip_address", ""):
            data += f'\nip_address: {request.findtext("ip_address", "")}'
        if request.findtext("time", ""):
            data += f'\ntime: {request.findtext("time", "")}'
        if request.findtext("return_code", ""):
            data += f'\nreturn_code: {request.findtext("return_code", "")}'
        if request.findtext("return_message", ""):
            data += f'\nreturn_message: {request.findtext("return_message", "")}'
        return data

    def getResponse(self):

        try:
            request = self.node.find('page/response')
            raw_data = request.find('raw_headers')
            data = raw_data.text
            if not data:
                data = self.contruct_request(request)
            data += self.construct_response(request)
            return data
        except:
            return 'None'


class System:

    def __init__(self, node, tag_exists):
        self.node = node
        if not tag_exists:
            self.user_agent = 'Arachni'
            self.url = self.getUrl()
            self.modules = ''
            self.version = self.node.find('version')
            self.start_time = self.node.find('start_datetime')
            self.finish_time = self.node.find('finish_datetime')
        else:
            self.user_agent = None
            self.url = None
            self.audited_elements = None
            self.modules = ''
            self.cookies = None
            self.getOptions()
            self.version = self.getDesc('version')
            self.start_time = self.getDesc('start_datetime')
            self.finish_time = self.getDesc('finish_datetime')
            self.note = self.getNote()

    def getUrl(self):
        sitemap = self.node.find("sitemap/entry")
        return sitemap.get('url')

    def getOptions(self):

        # Get values of options scan
        try:
            options = self.node.find('options')
        except:
            options = False
        if options:
            options_string = options.text
        else:
            options_string = None

        try:
            self.user_agent = self.node.find('user_agent').text
        except:
            self.user_agent = None

        self.url = self.node.find('url').text
        tags_audited_elements = self.node.find('audited_elements')
        element_text = []
        for element in tags_audited_elements:
            element_text.append(element.text)
        self.audited_elements = element_text
        tag_module = self.node.find('modules')
        module_text = []
        for module in tag_module:
            module_text.append(module.attrib['name'])
        self.modules = module_text
        self.cookies = self.node.find('cookies').text

    def getDesc(self, tag):

        # Return value of tag
        description = self.node.find(tag)

        if description and description.text:
            return description.text
        else:
            return None

    def getNote(self):
        result = ('Scan url:\n {} \nUser Agent:\n {} \nVersion Arachni:\n {} \nStart time:\n {} \nFinish time:\n {}'
                  '\nAudited Elements:\n {} \nModules:\n {} \nCookies:\n {}').format(self.url, self.user_agent,
                                                                                     self.version, self.start_time,
                                                                                     self.finish_time,
                                                                                     self.audited_elements,
                                                                                     self.modules, self.cookies)

        return result


class Plugins:
    """
    Support:
    WAF (Web Application Firewall) Detector (waf_detector)
    Healthmap (healthmap)
    """

    def __init__(self, plugins_node):

        self.plugins_node = plugins_node
        self.healthmap = self.getHealthmap()
        self.waf = self.getWaf()
        try:
            self.ip = plugins_node.find('resolver').find('results') \
                .find('hostname').get('ipaddress')
        except Exception:
            self.ip = None

    @staticmethod
    def get_value(name, node):
        x = node.find(name)
        if x:
            return x.text
        else:
            return ""

    def getHealthmap(self):

        # Get info about healthmap
        healthmap_tree = self.plugins_node.find('healthmap')
        if not healthmap_tree:
            return 'None'

        # Create urls list.
        list_urls = []
        map_results = healthmap_tree.find('results').find('map')

        for url in map_results:

            if url.tag == 'with_issues':
                list_urls.append(f"With Issues: {url.text}")
            else:
                list_urls.append(f"Without Issues: {url.text}")

        try:
            plugin_name = self.get_value('name', healthmap_tree)
            description = self.get_value('description', healthmap_tree)
            results = self.get_value('results', healthmap_tree)
            total = self.get_value('total', results)
            with_issues = self.get_value('with_issues', results)
            without_issues = self.get_value('without_issues', results)
            issue_percentage = self.get_value('issue_percentage', results)

            urls = '\n'.join(list_urls)
            result = (f"Plugin Name: {plugin_name}\nDescription: {description}\nStatistics:\nTotal: {total}"
                      f"\nWith Issues: {with_issues}\nWithout Issues: {without_issues}"
                      f"\nIssues percentage: {issue_percentage}\nResults Map:\n {urls}")
            return result

        except:
            return 'None'

    def getWaf(self):

        # Get info about waf plugin.
        waf_tree = self.plugins_node.find('waf_detector')
        try:
            plugin_name = self.get_value('name', waf_tree)
            description = self.get_value('description', waf_tree)
            results = waf_tree.find('results')
            message = self.get_value('message', results)
            status = self.get_value('status', results)
            result = (f"Plugin Name: {plugin_name}\nDescription: {description}\nResults:"
                      f"\nMessage: {message}\nStatus: {status}")
            return result
        except:
            return 'None'


class ArachniPlugin(PluginXMLFormat):

    # Plugin that parses Arachni's XML report files.

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["report", "arachni_report"]
        self.id = 'Arachni'
        self.name = 'Arachni XML Output Plugin'
        self.plugin_version = '1.0.1'
        self.version = '1.3.2'
        self.framework_version = '1.0.0'
        self.options = None
        self._command_regex = re.compile(r'^(arachni|\.\/arachni)\s+.*?')
        self.protocol = None
        self.hostname = None
        self.port = '80'
        self.address = None
        self._use_temp_file = True
        self._temp_file_extension = ["afr", "xml"]

    def report_belongs_to(self, **kwargs):
        if super().report_belongs_to(**kwargs):
            report_path = kwargs.get("report_path", "")
            with open(report_path) as f:
                output = f.read()
            return re.search("/Arachni/arachni/", output) is not None
        return False

    def _parse_filename(self, filename):
        """
        This plugin gets a dict of files, not just one file if it runs the command.
        We just need the xml.
        """
        if isinstance(filename, dict):
            filename = filename['xml']
        with open(filename, **self.open_options) as output:
            self.parseOutputString(output.read())
        if self._delete_temp_file:
            if isinstance(filename, dict):
                for _file in filename.values():
                    try:
                        os.remove(_file)
                    except Exception as e:
                        self.logger.error("Error on delete file: (%s) [%s]", _file, e)
            else:
                try:
                    os.remove(filename)
                except Exception as e:
                    self.logger.error("Error on delete file: (%s) [%s]", filename, e)

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.
        """
        parser = ArachniXmlParser(output)

        # Check xml parsed ok...
        if not parser.system:
            return

        self.hostname = self.getHostname(parser.system.url)
        if parser.plugins.ip:
            self.address = self.resolve_hostname(parser.plugins.ip)
        else:
            self.address = self.resolve_hostname(self.hostname)

        # Create host and interface
        host_id = self.createAndAddHost(self.address, hostnames=[self.hostname])

        # Create service
        service_id = self.createAndAddServiceToHost(
            host_id,
            self.protocol,
            'tcp',
            ports=[self.port],
            status='open',
            version='',
            description='')

        # Create issues.
        for issue in parser.issues:
            description = str(issue.description)
            resol = str(issue.remedy_guidance)

            references = issue.references
            if issue.cwe != 'None':
                cwe = ['CWE-' + str(issue.cwe)]
            if resol == 'None':
                resol = ''

            self.createAndAddVulnWebToService(
                host_id,
                service_id,
                name=issue.name,
                desc=description,
                data=issue.data,
                resolution=resol,
                ref=references,
                severity=issue.severity,
                website=self.hostname,
                path=issue.url,
                method=issue.method,
                pname=issue.var,
                params=issue.parameters,
                request=issue.request,
                response=issue.response,
                cwe=cwe
            )

        return

    def processCommandString(self, username, current_path, command_string):
        """
        Use bash to run sequentialy arachni and arachni_reporter
        """
        # Dont call the parent beacuse this plugin needs a different implementation
        if command_string.startswith("sudo"):
            params = " ".join(command_string.split()[2:])
        else:
            params = " ".join(command_string.split()[1:])
        self.vulns_data["command"]["params"] = params
        self.vulns_data["command"]["user"] = username
        self._output_file_path = {}
        self._delete_temp_file = True
        for ext in self._temp_file_extension:
            self._output_file_path[ext] = self._get_temp_file(extension=ext)
        afr_file_path = self._output_file_path['afr']
        xml_file_path = self._output_file_path['xml']
        report_arg_re = r"^.*(--report-save-path[=\s][^\s]+).*$"
        arg_match = re.match(report_arg_re, command_string)
        if arg_match is None:
            main_cmd = re.sub(r"(^.*?arachni)", r"\1 --report-save-path=%s" % afr_file_path, command_string)
        else:
            main_cmd = re.sub(arg_match.group(1), r"--report-save-path=%s" % afr_file_path, command_string)

        # add reporter
        cmd_prefix_match = re.match(r"(^.*?)arachni ", command_string)
        cmd_prefix = cmd_prefix_match.group(1)
        reporter_cmd = "{}{} --reporter=\"xml:outfile={}\" \"{}\"".format(cmd_prefix, "arachni_reporter", xml_file_path,
                                                                      afr_file_path)
        return "/usr/bin/env -- bash -c '{}  2>&1 && if [ -e \"{}\" ];then {} 2>&1;fi'".format(main_cmd,
                                                                                           afr_file_path,
                                                                                           reporter_cmd)

    def getHostname(self, url):

        # Strips protocol and gets hostname from URL.
        url_parse = urlparse(url)
        self.protocol = url_parse.scheme
        self.hostname = url_parse.netloc

        if self.protocol == 'https':
            self.port = 443
        elif self.protocol == 'http':
            if not self.port:
                self.port = 80

        return self.hostname


def createPlugin(*args, **kwargs):
    return ArachniPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from dateutil.parser import parse
import re
import json
from datetime import datetime
from dataclasses import dataclass
from faraday_plugins.plugins.plugin import PluginMultiLineJsonFormat

__author__ = "Nicolas Rebagliati"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Nicolas Rebagliati"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Nicolas Rebagliati"
__email__ = "nrebagliati@faradaysec.com"
__status__ = "Development"
CHECK_NUMBER_REGEX = re.compile(r"^(\[check\d\])")

@dataclass
class Issue:
    region: str
    profile: str
    severity: str
    scored: str
    account: str
    message: str
    control: str
    status: str
    level: str
    control_id: str
    timestamp: datetime
    compliance: str
    service: str
    caf_epic: str
    risk: str
    doc_link: str
    remediation: str
    resource_id: str


class ProwlerJsonParser:

    def parse_issues(self, records):
        for record in records:
            json_data = json.loads(record)
            region = json_data.get("Region", "AWS_REGION")
            profile = json_data.get("Profile", "")
            severity = json_data.get("Severity", "info").lower()
            scored = json_data.get("Status", "")
            account = json_data.get("Account Number", "")
            message = json_data.get("Message", "")
            control = CHECK_NUMBER_REGEX.sub("", json_data.get("Control", "")).strip()
            status = json_data.get("Status", "")
            level = json_data.get("Level", "")
            control_id = json_data.get("Control ID", "")
            timestamp = json_data.get("Timestamp", None)
            if timestamp:
                timestamp = parse(timestamp)
            compliance = json_data.get("Compliance", "")
            service = json_data.get("Service", "")
            caf_epic = [json_data.get("CAF Epic", "")]
            risk = json_data.get("Risk", "")
            doc_link = json_data.get("Doc link", "")
            remediation = json_data.get("Remediation", "")
            resource_id = json_data.get("Resource ID", "")
            if status == "FAIL":
                self.issues.append(Issue(region=region, profile=profile, severity=severity, scored=scored,
                                         account=account, message=message, control=control, status=status,
                                         level=level, control_id=control_id, timestamp=timestamp, compliance=compliance,
                                         service=service, caf_epic=caf_epic, risk=risk, doc_link=doc_link,
                                         remediation=remediation, resource_id=resource_id))

    def __init__(self, json_output):
        self.issues = []
        self.parse_issues(json_output.splitlines())


class ProwlerPlugin(PluginMultiLineJsonFormat):
    """ Handle the AWS Prowler tool. Detects the output of the tool
    and adds the information to Faraday.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "prowler"
        self.name = "Prowler"
        self.plugin_version = "0.1"
        self.version = "0.0.1"
        self.json_keys = {"Profile", "Account Number", "Region"}

    def parseOutputString(self, output, debug=False):
        parser = ProwlerJsonParser(output)
        for issue in parser.issues:
            host_name = f"{issue.service}-{issue.account}-{issue.region}"
            host_id = self.createAndAddHost(name=host_name,
                                            description=f"AWS Service: {issue.service} - Account: {issue.account}"
                                                        f" - Region: {issue.region}")

            vuln_desc = f"{issue.risk}\nCompliance: {issue.compliance}\nMessage: {issue.message}"
            self.createAndAddVulnToHost(host_id=host_id, name=issue.control, desc=vuln_desc,
                                        data=f"Resource ID: {issue.resource_id}",
                                        severity=self.normalize_severity(issue.severity), resolution=issue.remediation,
                                        run_date=issue.timestamp, external_id=f"{self.name.upper()}-{issue.control_id}",
                                        ref=[issue.doc_link],
                                        policyviolations=issue.caf_epic)


def createPlugin(*args, **kwargs):
    return ProwlerPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
from collections import defaultdict

from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class DnsmapParser:
    """
    The objective of this class is to parse an xml file generated by the
    dnsmap tool.

    TODO: Handle errors.
    TODO: Test dnsmap output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param dnsmap_filepath A proper simple report generated by dnsmap
    """

    def __init__(self, output):
        self.items = defaultdict(list)
        if "\n\n" in output:
            self.parse_txt(output)
        else:
            self.parse_csv(output)

    def parse_txt(self, output):
        hosts = self.split_output_lines(output)

        for host_data in hosts:
            if len(host_data) == 2:
                ip = self.clean_ip(host_data[1])
                hostname = host_data[0]
                self.add_host_info_to_items(ip, hostname)
            elif len(host_data) > 2:
                hostname = host_data.pop(0)
                for ip_address in host_data:
                    ip = self.clean_ip(ip_address)
                    self.add_host_info_to_items(ip, hostname)

    def parse_csv(self, output):
        hosts = list(filter(None, output.splitlines()))

        for host in hosts:
            host_data = host.split(",", 1)
            if host_data[1].count(',') == 0:
                ip = host_data[1]
                hostname = host_data[0]
                self.add_host_info_to_items(ip, hostname)
            else:
                hostname = host_data.pop(0)
                ips = host_data[0].split(",")
                for ip_address in ips:
                    self.add_host_info_to_items(ip_address, hostname)

    def split_output_lines(self, output):
        splitted = output.splitlines()
        hosts_list = []
        aux_list = []
        for i in range(0, len(splitted)):
            if not splitted[i]:
                hosts_list.append(aux_list)
                aux_list = []
                continue
            else:
                aux_list.append(splitted[i])
        return hosts_list

    def clean_ip(self, item):
        ip = item.split(':', 1)
        return ip[1].strip()

    def add_host_info_to_items(self, ip_address, hostname):
        self.items[ip_address].append(hostname)


class DnsmapPlugin(PluginBase):
    """Example plugin to parse dnsmap output."""

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Dnsmap"
        self.name = "Dnsmap Output Plugin"
        self.plugin_version = "0.3"
        self.version = "0.30"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^(sudo dnsmap|dnsmap|\.\/dnsmap)\s+.*?')
        self.xml_arg_re = re.compile(r"^.*(-r\s*[^\s]+).*$")
        self._use_temp_file = True
        self._temp_file_extension = "txt"

    def canParseCommandString(self, current_input):
        if self._command_regex.match(current_input.strip()):
            self.command = self.get_command(current_input)
            return True
        else:
            return False

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.
        """
        parser = DnsmapParser(output)
        for ip_address, hostnames in parser.items.items():
            h_id = self.createAndAddHost(ip_address, hostnames=hostnames)
        return True

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the parameter to get output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        arg_match = self.xml_arg_re.match(command_string)

        if arg_match is None:
            return f"{command_string} -r {self._output_file_path} \\n"
        else:
            return re.sub(arg_match.group(1),
                          r"-r %s" % self._output_file_path,
                          command_string)


def createPlugin(*args, **kwargs):
    return DnsmapPlugin(*args, **kwargs)



from typing import List
import re

from faraday_plugins.plugins.plugins_utils import CVE_regex

CVE_WITH_P_regex = re.compile(r'\(CVE-\d{4}-\d{4,7}\)')


class Service:
    def __init__(self, node):
        self.node = node

    @property
    def name(self):
        return self.node.get("name", "")

    @property
    def port(self):
        return self.node.get("port", "")

    @property
    def protocol(self):
        return self.node.get("transport", "")

    @property
    def status(self) -> str:
        return self.node.get("status", "")


class Host:
    def __init__(self, node):
        self.node = node

    @property
    def host_id(self) -> str:
        return self.node.get("id", "")

    @property
    def hostname(self) -> str:
        return self.node.get("hostname", "")

    @property
    def name(self) -> str:
        return self.node.get("ip", "")

    @property
    def os(self) -> str:
        return self.node.get("os_name", "")

    @property
    def services(self) -> List[Service]:
        return [Service(ser) for ser in self.node.get("services", "")]


class Vulneravility:
    def __init__(self, node):
        self.node = node

    @property
    def external_id(self) -> str:
        return "Pentera-"+self.node.get("id", "")

    @property
    def name(self) -> str:
        return CVE_WITH_P_regex.sub("", self.node.get("name", "")).strip()

    @property
    def description(self) -> str:
        desc = self.node.get("summary", "")
        if not desc:
            desc = self.name
        return desc

    @property
    def found_on(self) -> str:
        return self.node.get("found_on", "")

    @property
    def host(self) -> str:
        return self.node.get("target", "")

    @property
    def host_id(self) -> str:
        return self.node.get("target_id", "")

    @property
    def port(self) -> str:
        return self.node.get("port", "")

    @property
    def protocol(self) -> str:
        return self.node.get("protocol", "")

    @property
    def severity(self) -> float:
        return float(self.node.get("severity", 0))

    @property
    def data(self) -> str:
        return self.node.get("insight", "")

    @property
    def resolution(self) -> str:
        return self.node.get("remediation", "")

    @property
    def cve(self) -> str:
        return CVE_regex.sub("", self.node.get("name", "")).strip()
class Achievment:
    def __init__(self, node):
        self.node = node


class Meta:
    def __init__(self, node):
        self.node = node


class PenteraJsonParser:
    def __init__(self, node):
        self.node = node

    @property
    def meta(self) -> Meta:
        return Meta(self.node.get('meta', {}))

    @property
    def achievements(self) -> List[Achievment]:
        return [Achievment(i) for i in self.node.get('Achievement', [])]

    @property
    def vulneravilities(self) -> List[Vulneravility]:
        return [Vulneravility(i) for i in self.node.get('vulnerabilities', [])]

    @property
    def hosts(self) -> List[Host]:
        return [Host(i) for i in self.node.get('hosts')]



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Facundo de Guzmán, Esteban Guillardoy"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Facundo de Guzmán", "Esteban Guillardoy"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Facundo de Guzmán"
__email__ = "fdeguzman@ribadeohacklab.com.ar"
__status__ = "Development"


class TelnetRouterPlugin(PluginBase):
    """
    This plugin handles telnet command.
    Basically detects if user was able to connect to a device
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Telnet"
        self.name = "Telnet"
        self.plugin_version = "0.0.1"
        self.version = "0.17"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^telnet\s+.*?')
        self._host_ip = None
        self._host = []
        self._port = "23"
        self._completition = {
            "": "telnet [-468ELadr] [-S tos] [-b address] [-e escapechar] [-l user] [-n tracefile] [host [port]]",
            "-4": "Force IPv4 address resolution.",
            "-6": "Force IPv6 address resolution.",
            "-8": "Request 8-bit operation. This causes an attempt to negotiate the TELNET BINARY option for both input and output. By default telnet is not 8-bit clean.",
            "-E": "Disables the escape character functionality; that is, sets the escape character to ``no character''.",
            "-L": "Specifies an 8-bit data path on output.  This causes the TELNET BINARY option to be negotiated on just output.",
            "-a": "Attempt automatic login.  Currently, this sends the user name via the USER variable of the ENVIRON option if supported by the remote system. The username is retrieved via getlogin(3).",
            "-b": "-b &lt;address&gt; Use bind(2) on the local socket to bind it to a specific local address.",
            "-d": "Sets the initial value of the debug toggle to TRUE.",
            "-r": "Emulate rlogin(1).  In this mode, the default escape character is a tilde. Also, the interpretation of the escape character is changed: an escape character followed by a dot causes telnet to disconnect from the remote host. A ^Z instead of a dot suspends telnet, and a ^] (the default telnet escape character) generates a normal telnet prompt. These codes are accepted only at the beginning of a line.",
            "-S": "-S &lt;tos&gt;  Sets the IP type-of-service (TOS) option for the telnet connection to the value tos.",
            "-e": "-e &lt;escapechar&gt; Sets the escape character to escapechar. If no character is supplied, no escape character will be used.  Entering the escape character while connected causes telnet to drop to command mode.",
            "-l": "-l &lt;user&gt; Specify user as the user to log in as on the remote system. This is accomplished by sending the specified name as the USER environment variable, so it requires that the remote system support the TELNET ENVIRON option. This option implies the -a option, and may also be used with the open command.",
            "-n": "-n &lt;tracefile&gt; Opens tracefile for recording trace information.  See the set tracefile command below.",
        }



    def parseOutputString(self, output):

        host_info = re.search(r"Connected to (.+)\.", output)

        hostname = host_info.group(1)
        ip_address = self.resolve_hostname(hostname)

        if host_info is not None:
            h_id = self.createAndAddHost(ip_address, hostnames=[hostname])
            s_id = self.createAndAddServiceToHost(h_id, self._port, "tcp", ports=[self._port], status="open")
        return True

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        count_args = command_string.split()
        c = count_args.__len__()
        self._port = "23"
        if re.search(r"[\d]+", count_args[c - 1]):
            self._port = count_args[c - 1]


def createPlugin(*args, **kwargs):
    return TelnetRouterPlugin(*args, **kwargs)



#!/usr/bin/env python
"""
Faraday Penetration Test IDE
Copyright (C) 2016  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import xml.etree.ElementTree as ET
from datetime import datetime

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = 'Blas Moyano'
__copyright__ = 'Copyright 2020, Faraday Project'
__credits__ = ['Blas Moyano']
__license__ = ''
__version__ = '1.0.0'
__status__ = 'Development'


class AppSpiderParser:
    def __init__(self, xml_output):
        self.tree = self.parse_xml(xml_output)
        if self.tree:
            self.vuln_list = self.tree.find('VulnList')
            self.name_scan = self.tree.findtext('ScanName')
        else:
            self.tree = None

    @staticmethod
    def parse_xml(xml_output):
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f'SyntaxError In xml: {err}. {xml_output}')
            return None
        return tree


class AppSpiderPlugin(PluginXMLFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["VulnSummary"]
        self.id = 'AppSpider'
        self.name = 'AppSpider XML Output Plugin'
        self.plugin_version = '1.0.0'
        self.version = '1.0.0'
        self.framework_version = '1.0.0'
        self.options = None
        self.protocol = None
        self.port = '80'
        self.address = None

    def parseOutputString(self, output):
        parser = AppSpiderParser(output)
        websites = []
        websites_ip = []

        for vuln in parser.vuln_list:
            websites.append(vuln.find('WebSite').text)
            websites_ip.append(vuln.find('WebSiteIP').text)

        url = set(websites)
        ip = set(websites_ip)
        if None in ip:
            ip.remove(None)

        host_id = self.createAndAddHost(name=sorted(list(ip))[0], hostnames=sorted(list(url)),
                                        description=parser.name_scan)
        data_info = []

        for vulns in parser.vuln_list:

            vuln_name = vulns.findtext('VulnType')
            vuln_desc = vulns.findtext('Description')
            vuln_ref = vulns.findtext('VulnUrl')
            severity = vulns.findtext('AttackScore')
            vuln_resolution = vulns.findtext('Recommendation')
            vuln_external_id = vulns.findtext('DbId')
            vuln_run_date = vulns.findtext('ScanDate')
            data_info.append(vulns.findtext('AttackClass'))
            cwe = ["CWE-" + vulns.findtext('CweId')] if vulns.findtext('CweId') else []
            data_info.append(vulns.findtext('CAPEC'))
            data_info.append(vulns.findtext('DISSA_ASC'))
            data_info.append(vulns.findtext('OWASP2007'))
            data_info.append(vulns.findtext('OWASP2010'))
            data_info.append(vulns.findtext('OWASP2013'))
            data_info.append(vulns.findtext('OVAL'))
            data_info.append(vulns.findtext('WASC'))

            if severity == '1-Informational':
                severity = 0
            elif severity == '2-Low':
                severity = 1
            elif severity == '3-Medium':
                severity = 2
            elif severity == '4-High':
                severity = 3
            else:
                severity = 10

            str_data = f'AttackClass: {data_info[0]}, CAPEC: {data_info[1]}, ' \
                       f'DISSA_ASC: {data_info[2]}, OWASP2007: {data_info[3]}, OWASP2010: {data_info[4]}, ' \
                       f'OWASP2013: {data_info[5]}, OVAL: {data_info[6]}, WASC: {data_info[7]}'

            if vuln_run_date is None:
                vuln_run_date = None
            else:
                vuln_run_date = datetime.strptime(vuln_run_date, '%Y-%m-%d %H:%M:%S')

            self.createAndAddVulnToHost(host_id=host_id, name=vuln_name, desc=vuln_desc, ref=[vuln_ref],
                                        severity=severity, resolution=vuln_resolution, run_date=vuln_run_date,
                                        external_id=vuln_external_id, data=str_data, cwe=cwe)


def createPlugin(*args, **kwargs):
    return AppSpiderPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import json
from faraday_plugins.plugins.plugin import PluginJsonFormat

__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Blas Moyano"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"


class SslyzeJsonParser:

    def __init__(self, json_output, resolve_hostname):
        self.resolve_hostname = resolve_hostname
        json_sslyze = json.loads(json_output)
        scan_result = json_sslyze.get('server_scan_results')
        self.list_vul = self.get_vuln(scan_result)

    def get_vuln(self, scan_result):
        list_vuln = []
        if scan_result:
            for scan in scan_result:
                server_info = scan.get('server_info', scan).get('server_location')
                host = self.get_host(server_info) if server_info else {}

                scan_commands_results = scan.get('scan_commands_results', scan.get("scan_result", {}))

                if len(scan_commands_results) > 0:
                    commands = []
                    for command in scan_commands_results:
                        if command.find("cipher") >= 0 and scan_commands_results[command].get('result', True):
                            commands.append(command)
                    ciphers = self.get_cipher(scan_commands_results, commands)
                else:
                    ciphers = {}


                certificate_info = scan_commands_results.get('certificate_info')
                certif = self.get_certification(certificate_info) if certificate_info else {}

                heartbleed_reulsts = scan_commands_results.get('heartbleed')
                heartbleed = self.get_heartbleed(heartbleed_reulsts) if heartbleed_reulsts else {}

                openssl_ccs_injection = scan_commands_results.get('openssl_ccs_injection')
                openssl_ccs = self.get_openssl_ccs(openssl_ccs_injection) if openssl_ccs_injection else {}
                json_vuln = {
                    "host_info": host,
                    "certification": certif,
                    "ciphers": ciphers,
                    "heartbleed": heartbleed,
                    "openssl_ccs": openssl_ccs
                }

                list_vuln.append(json_vuln)
        return list_vuln

    def get_host(self, server_location):
        port = server_location.get('port', None)
        hostname = server_location.get('hostname', None)
        ip = server_location.get('ip_address', self.resolve_hostname(hostname))
        if port != 443:
            url = f"https://{hostname}:{port}"
        else:
            url = f"https://{hostname}"

        json_host = {
            "name": 'https',
            "ip": ip,
            "hostname": hostname,
            "port": port,
            "protocol": 'tcp',
            "url": url
        }

        return json_host

    def get_certification(self, certificate):
        certif_deploy = certificate.get('certificate_deployments', certificate.get('result'))
        certif_deploy = certif_deploy.get('certificate_deployments', [{}]) if isinstance(certif_deploy, dict) else [{}]
        send_certif = certif_deploy[0].get('leaf_certificate_subject_matches_hostname', True)

        if not send_certif:
            subject = certif_deploy[0]['received_certificate_chain'][0]['subject']
            why = subject.get('rfc4514_string')
            if not why:
                why = subject.get('attributes', {}).get('rfc4514_string')
            hostname_used_for_server = certificate.get('hostname_used_for_server_name_indication', certificate.get("result", {})
                                                       .get('hostname_used_for_server_name_indication', 'Not hostname'))
            json_certif = {
                "name": "SSL/TLS Certificate Mismatch",
                "desc": "The software communicates with a host that provides a certificate, but the software does not properly ensure that the certificate is actually associated with that host.",
                "data": f"Certificate {why} does not match server hostname {hostname_used_for_server}",
                "impact": {"integrity": True},
                "ref": ["https://cwe.mitre.org/data/definitions/297.html"],
                "external_id": "CWE-297",
                "severity": "low"
            }
        else:
            json_certif = {}
        return json_certif

    def get_cipher(self, scan_result, list_commands):
        weak_cipher_list = [
            "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA",
            "TLS_RSA_WITH_AES_128_CBC_SHA",
            "TLS_RSA_WITH_AES_128_CBC_SHA256",
            "TLS_RSA_WITH_AES_128_GCM_SHA256",
            "TLS_RSA_WITH_AES_256_CBC_SHA",
            "TLS_RSA_WITH_AES_256_CBC_SHA256",
            "TLS_RSA_WITH_AES_256_GCM_SHA384",
            "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
            "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA",
            "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA"
        ]
        weak_cipher = {}
        for command in list_commands:
            weak_cipher[command] = []
            try:
                if scan_result[command]['accepted_cipher_suites']:
                    for cipher_suite in scan_result[command]['accepted_cipher_suites']:
                        name_cipher = cipher_suite['cipher_suite'].get('name')
                        if name_cipher in weak_cipher_list:
                            if name_cipher not in weak_cipher[command]:
                                weak_cipher[command].append(name_cipher)
            except KeyError:
                pass

            try:
                if scan_result[command]['cipher_suite_preferred_by_server'] is not None:
                    for cipher_suite in scan_result[command]['accepted_cipher_suites']:
                        name_cipher = cipher_suite['cipher_suite'].get('name')
                        if name_cipher in weak_cipher_list:
                            if name_cipher not in weak_cipher[command]:
                                weak_cipher[command].append(name_cipher)
            except KeyError:
                pass

        return weak_cipher

    def get_heartbleed(self, heartbleed):
        json_heartbleed = {}
        vulnerable_to_heartbleed = heartbleed.get('is_vulnerable_to_heartbleed', heartbleed.get('result', False))
        vulnerable_to_heartbleed = vulnerable_to_heartbleed.get('is_vulnerable_to_heartbleed') if vulnerable_to_heartbleed else False
        if vulnerable_to_heartbleed:
            json_heartbleed = {
                "name": "OpenSSL Heartbleed",
                "desc": "OpenSSL versions 1.0.1 through 1.0.1f contain a flaw in its implementation of the TLS/DTLS heartbeat functionality. This flaw allows an attacker to retrieve private memory of an application that uses the vulnerable OpenSSL library in chunks of 64k at a time. Note that an attacker can repeatedly leverage the vulnerability to retrieve as many 64k chunks of memory as are necessary to retrieve the intended secrets. The sensitive information that may be retrieved using this vulnerability include:\n\n Primary key material (secret keys)\n Secondary key material (user names and passwords used by vulnerable services)\n Protected content (sensitive data used by vulnerable services)\n Collateral (memory addresses and content that can be leveraged to bypass exploit mitigations)\n Exploit code is publicly available for this vulnerability.  Additional details may be found in CERT/CC Vulnerability Note VU#720951.",
                "impact": {"confidentiality": True},
                "ref": ["https://nvd.nist.gov/vuln/detail/CVE-2014-0160", "https://heartbleed.com/", "https://us-cert.cisa.gov/ncas/alerts/TA14-098A"],
                "external_id": "CVE-2014-0160",
                "severity": "high"
            }
        return json_heartbleed

    def get_openssl_ccs(self, openssl_ccs):
        json_openssl_ccs = {}
        is_vulnerable_to_ccs_injection = openssl_ccs.get('is_vulnerable_to_ccs_injection', openssl_ccs.get('result', False))
        is_vulnerable_to_ccs_injection = is_vulnerable_to_ccs_injection.get('is_vulnerable_to_ccs_injection', False) if is_vulnerable_to_ccs_injection else False
        if is_vulnerable_to_ccs_injection:
            json_openssl_ccs = {
                "name": "OpenSSL CCS Injection",
                "desc": 'OpenSSL before 0.9.8za, 1.0.0 before 1.0.0m, and 1.0.1 before 1.0.1h does not properly restrict processing of ChangeCipherSpec messages, which allows man-in-the-middle attackers to trigger use of a zero-length master key in certain OpenSSL-to-OpenSSL communications, and consequently hijack sessions or obtain sensitive information, via a crafted TLS handshake, aka the "CCS Injection" vulnerability."',
                "impact": {"confidentiality": True, "integrity": True},
                "ref": ["http://ccsinjection.lepidum.co.jp/", "https://nvd.nist.gov/vuln/detail/CVE-2014-0224"],
                "external_id": "CVE-2014-0224",
                "severity": "high"
            }
        return json_openssl_ccs


class SslyzePlugin(PluginJsonFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Sslyze_JSON"
        self.name = "Sslyze Json"
        self.plugin_version = "0.1"
        self.version = "3.4.5"
        self.json_keys = {'server_scan_results', 'sslyze_url', 'sslyze_version'}
        self._command_regex = re.compile(r'^(sudo sslyze|sslyze|\.\/sslyze)\s+.*?')
        self.json_arg_re = re.compile(r"^.*(--json_out\s*[^\s]+).*$")
        self._use_temp_file = True
        self._temp_file_extension = "json"

    def parseOutputString(self, output):
        parser = SslyzeJsonParser(output, self.resolve_hostname)

        for info_sslyze in parser.list_vul:
            info_sslyze['host_info'].get('hostname')
            host_id = self.createAndAddHost(
                info_sslyze['host_info'].get('ip'),
                os="unknown",
                hostnames=[
                    info_sslyze['host_info'].get('hostname')
                ]
            )
            service_id = self.createAndAddServiceToHost(
                host_id,
                name=info_sslyze['host_info'].get('name'),
                protocol=info_sslyze['host_info'].get('protocol'),
                ports=[
                    info_sslyze['host_info'].get('port')
                ]
            )

            if info_sslyze['certification']:
                self.createAndAddVulnWebToService(
                    host_id,
                    service_id,
                    name=info_sslyze['certification'].get('name'),
                    desc=info_sslyze['certification'].get('desc'),
                    data=info_sslyze['certification'].get('data'),
                    impact=info_sslyze['certification'].get('impact'),
                    ref=info_sslyze['certification'].get('ref'),
                    easeofresolution="trivial",
                    external_id=info_sslyze['certification'].get('external_id'),
                    website=info_sslyze['host_info'].get('url'),
                    severity=info_sslyze['certification'].get('severity'))

            cipherlist = []
            if info_sslyze['ciphers']:
                for k, v in info_sslyze['ciphers'].items():
                    if len(v) != 0:
                        for ciphers in v:
                            key = k.replace('_cipher_suites', '')
                            cipherlist.append(f"In protocol [{key}], weak cipher suite: {ciphers}")
                if cipherlist:
                    self.createAndAddVulnWebToService(
                        host_id,
                        service_id,
                        name="SSL/TLS Weak Cipher Suites Supported",
                        desc="The software stores or transmits sensitive data using an encryption scheme that is theoretically sound, but is not strong enough for the level of protection required.",
                        data="\n".join(cipherlist),
                        impact={"confidentiality": True},
                        ref=["https://cwe.mitre.org/data/definitions/326.html"],
                        easeofresolution="trivial",
                        external_id="CWE-326",
                        website=info_sslyze['host_info'].get('url'),
                        severity="low")

            if info_sslyze['heartbleed']:
                self.createAndAddVulnWebToService(
                    host_id,
                    service_id,
                    name=info_sslyze['heartbleed'].get('name'),
                    desc=info_sslyze['heartbleed'].get('desc'),
                    impact=info_sslyze['heartbleed'].get('impact'),
                    ref=info_sslyze['heartbleed'].get('ref'),
                    easeofresolution="trivial",
                    external_id=info_sslyze['heartbleed'].get('external_id'),
                    website=info_sslyze['host_info'].get('url'),
                    severity=info_sslyze['heartbleed'].get('severity'))

            if info_sslyze['openssl_ccs']:
                self.createAndAddVulnWebToService(
                    host_id,
                    service_id,
                    name=info_sslyze['openssl_ccs'].get('name'),
                    desc=info_sslyze['openssl_ccs'].get('desc'),
                    impact=info_sslyze['openssl_ccs'].get('impact'),
                    ref=info_sslyze['openssl_ccs'].get('ref'),
                    easeofresolution="trivial",
                    external_id=info_sslyze['openssl_ccs'].get('external_id'),
                    website=info_sslyze['host_info'].get('url'),
                    severity=info_sslyze['openssl_ccs'].get('severity'))

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        arg_match = self.json_arg_re.match(command_string)
        if arg_match is None:
            return re.sub(r"(^.*?sslyze)",
                          r"\1 --json_out %s" % self._output_file_path,
                          command_string)
        else:
            return re.sub(arg_match.group(1),
                          r"--json_out %s" % self._output_file_path,
                          command_string)


def createPlugin(*args, **kwargs):
    return SslyzePlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re

from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Facundo de Guzmán, Esteban Guillardoy"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Facundo de Guzmán", "Esteban Guillardoy"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class CmdPingPlugin(PluginBase):
    """
    This plugin handles ping command.
    Basically detects if user was able to connect to a device
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "ping"
        self.name = "Ping"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self._command_regex = re.compile(r'^(sudo ping|ping|sudo ping6|ping6)\s+.*?')

    def parseOutputString(self, output):

        reg = re.search(r"PING ([\w\.-:]+)( |)\(([\w\.:]+)\)", output)
        if re.search("0 received|unknown host", output) is None and reg is not None:
            ip_address = reg.group(3)
            hostname = reg.group(1)
            self.createAndAddHost(ip_address, hostnames=[hostname])
        return True

    def _isIPV4(self, ip):
        if len(ip.split(".")) == 4:
            return True
        else:
            return False


def createPlugin(*args, **kwargs):
    return CmdPingPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

from faraday_plugins.plugins.plugin import PluginCSVFormat
import csv
import io


__author__ = "Gonzalo Martinez"
__copyright__ = "Copyright (c) 2019, Infobyte LLC"
__credits__ = ["Gonzalo Martinez"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Gonzalo Martinez"
__email__ = "gmartinez@infobytesec.com"
__status__ = "Development"


class NessusScPlugin(PluginCSVFormat):
    """
    Example plugin to parse Nessus Sc output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.csv_headers = [{'Plugin', 'Plugin Name'}]
        self.id = "Nessus_sc"
        self.name = "Nessus Sc Output Plugin"
        self.plugin_version = "1.0.0"
        self.version = "1.0.0"
        self.framework_version = "1.0.0"

    def parseOutputString(self, output):
        try:
            reader = csv.DictReader(io.StringIO(output))
        except:
            print("Error parser output")
            return None

        for row in reader:
            ip = row['IP Address']
            hostname = row['DNS Name']
            h_id = self.createAndAddHost(name=ip, hostnames=hostname)
            protocol = row['Protocol']
            port = row['Port']
            s_id = self.createAndAddServiceToHost(h_id, name=port, protocol=protocol, ports=port, status="open")
            name = row['Plugin Name']
            severity = row['Severity']
            description = row['Description']
            vuln = {"name": name, "severity": severity, "desc": description}
            solution = row['Solution']
            if solution:
                vuln["resolution"] = solution
            cvss3_vector = row['CVSS V3 Vector']
            if cvss3_vector:
                if not cvss3_vector.startswith("CVSS:3.0/"):
                    cvss3_vector = "CVSS:3.0/"+cvss3_vector
                vuln["cvss3"] = {"vector_string": cvss3_vector}
            cvss2_vector = row['CVSS V2 Vector']
            if cvss2_vector:
                vuln["cvss2"] = {"vector_string": cvss2_vector}
            external_ref = row["See Also"]
            cross_ref = row["Cross References"]
            references = []
            if external_ref:
                references.append(external_ref)
            if cross_ref:
                references.append(cross_ref)
            vuln["ref"] = references
            cve = row['CVE']
            if cve:
                vuln["cve"] = cve
            self.createAndAddVulnToService(h_id, s_id, **vuln)


def createPlugin(*args, **kwargs):
    return NessusScPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
import xml.etree.ElementTree as ET

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = 'Francisco Amato'
__copyright__ = 'Copyright (c) 2013, Infobyte LLC'
__credits__ = ['Francisco Amato']
__license__ = ''
__version__ = '1.0.0'
__maintainer__ = 'Francisco Amato'
__email__ = 'famato@infobytesec.com'
__status__ = 'Development'


def cleaner_results(string):
    try:
        result = string.replace('<P>', '').replace('<UL>', ''). \
            replace('<LI>', '').replace('<BR>', ''). \
            replace('<A HREF="', '').replace('</A>', ' '). \
            replace('" TARGET="_blank">', ' ').replace('&quot;', '"')
        return result
    except:
        return ''


class QualysguardXmlParser():
    """
    The objective of this class is to parse an xml file generated by
    the qualysguard tool.

    TODO: Handle errors.
    TODO: Test qualysguard output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param qualysguard_xml_filepath A proper xml generated by qualysguard
    """

    def __init__(self, xml_output):
        tree, type_report = self.parse_xml(xml_output)

        if not tree or type_report is None:
            self.items = []
            return

        if type_report == 'ASSET_DATA_REPORT':
            self.items = [data for data in self.get_items_asset_report(tree)]
        elif type_report == 'SCAN':
            self.items = [data for data in self.get_items_scan_report(tree)]

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead
        of reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """

        asset_data_report = '<!DOCTYPE ASSET_DATA_REPORT SYSTEM'
        scan_report = '<!DOCTYPE SCAN SYSTEM'

        try:
            tree = ET.fromstring(xml_output)

            if asset_data_report in xml_output:
                type_report = 'ASSET_DATA_REPORT'
            elif scan_report in xml_output:
                type_report = 'SCAN'
            else:
                type_report = None

        except SyntaxError as err:
            return None, None

        return tree, type_report

    def get_items_scan_report(self, tree):
        """
        @return items A list of Host instances
        """
        for node in tree.findall('IP'):
            yield ItemScanReport(node)

    def get_items_asset_report(self, tree):
        """
        @return items A list of Host instances
        """
        for node in tree.find('HOST_LIST').findall('HOST'):
            yield ItemAssetReport(node, tree)


class ItemAssetReport():
    """
    An abstract representation of a Item (HOST) for a Asset Report.
    @param item_node A item_node taken from an qualysguard xml tree
    """

    def __init__(self, item_node, tree):

        self.node = item_node
        self.ip = self.get_text_from_subnode('IP')
        self.hostname = self.get_text_from_subnode('DNS') or ''
        self.os = self.get_text_from_subnode('OPERATING_SYSTEM')
        self.vulns = self.getResults(tree)

    def getResults(self, tree):
        glossary = tree.find('GLOSSARY/VULN_DETAILS_LIST')
        for self.issue in self.node.find('VULN_INFO_LIST'):
            yield ResultsAssetReport(self.issue, glossary)

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None


class ResultsAssetReport():
    """
    A abstraction of Results for a Asset Report of Qualysguard.
    """

    def __init__(self, issue_node, glossary):

        # VULN_INFO ElementTree
        self.node = issue_node
        self.port = self.get_text_from_subnode(self.node, 'PORT')
        self.protocol = self.get_text_from_subnode(self.node, 'PROTOCOL')
        self.name = self.get_text_from_subnode(self.node, 'QID')
        self.external_id = self.name
        self.result = self.get_text_from_subnode(self.node, 'RESULT')

        self.severity_dict = {
            '1': 'info',
            '2': 'low',
            '3': 'med',
            '4': 'high',
            '5': 'critical'}

        # GLOSSARY TAG
        self.glossary = glossary
        self.severity = self.severity_dict.get(self.get_text_from_glossary('SEVERITY'), 'info')
        self.title = self.get_text_from_glossary('TITLE')
        self.cvss2 = {}
        self.pci = self.get_text_from_glossary('PCI_FLAG')
        self.solution = self.get_text_from_glossary('SOLUTION')
        self.impact = self.get_text_from_glossary('IMPACT')

        # Description
        self.desc = cleaner_results(self.get_text_from_glossary('THREAT'))
        if not self.desc:
            self.desc = ''
        if self.result:
            self.desc += '\n\nResult: ' + cleaner_results(self.result)
        if self.impact:
            self.desc += '\n\nImpact: ' + cleaner_results(self.impact)
        if self.result:
            self.desc += '\n\nSolution: ' + cleaner_results(self.solution)

        # References
        self.ref = []
        self.cve = []

        cve_id = self.get_text_from_glossary('CVE_ID_LIST/CVE_ID/ID')
        if cve_id:
            self.cve.append(cve_id)

        if self.pci:
            self.ref.append(f'PCI: {self.pci}')

    def get_text_from_glossary(self, tag):
        """
        Finds a subnode in the glossary and retrieves a value of this.
        Filter by QualysId.

        @return An attribute value
        """

        for vuln_detail in self.glossary:
            id_act = vuln_detail.get('id').strip('qid_')
            if id_act == self.name:
                text = vuln_detail.find(tag)
                if text is not None:
                    return text.text
                else:
                    return None

    def get_text_from_subnode(self, node, subnode_xpath_expr):
        """
        Finds a subnode in the node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text
        return None


class ItemScanReport():
    """
    An abstract representation of a Item for a 'SCAN' report of Qualysguard.

    @param item_node A item_node taken from an qualysguard xml tree
    """

    def __init__(self, item_node):
        self.node = item_node
        self.ip = item_node.get('value')
        self.os = self.get_text_from_subnode('OS')
        self.hostname = self.get_hostname(item_node)
        self.vulns = self.getResults(item_node)

    def getResults(self, tree):
        """
        :param tree:
        """
        for self.issues in tree.findall('VULNS/CAT'):
            for v in self.issues.findall('VULN'):
                yield ResultsScanReport(v, self.issues)
        for self.issues in tree.findall('INFOS/CAT'):
            for v in self.issues.findall('INFO'):
                yield ResultsScanReport(v, self.issues)
        for self.issues in tree.findall('SERVICES/CAT'):
            for v in self.issues.findall('SERVICE'):
                yield ResultsScanReport(v, self.issues)
        for self.issues in tree.findall('PRACTICES/CAT'):
            for v in self.issues.findall('PRACTICE'):
                yield ResultsScanReport(v, self.issues)

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None

    def get_hostname(self, node):
        hostname = node.get('name')

        if hostname == 'No registered hostname':
            return ""

        return hostname


class ResultsScanReport():
    """
    An abstraction of Result for Qualysguard 'SCAN' Report.
    """

    def __init__(self, issue_node, parent):
        self.node = issue_node
        self.port = parent.get('port')
        self.protocol = parent.get('protocol')
        self.name = self.node.get('number')
        self.external_id = self.node.get('number')
        self.title = self.get_text_from_subnode('TITLE')
        self.cvss2 = {}

        self.diagnosis = self.get_text_from_subnode('DIAGNOSIS')
        self.solution = self.get_text_from_subnode('SOLUTION')
        self.result = self.get_text_from_subnode('RESULT')
        self.consequence = self.get_text_from_subnode('CONSEQUENCE')

        self.severity_dict = {
            '1': 'info',
            '2': 'low',
            '3': 'med',
            '4': 'high',
            '5': 'critical'}

        self.severity = self.severity_dict.get(self.node.get('severity'), 'info')

        self.desc = cleaner_results(self.diagnosis)
        if self.result:
            self.desc += '\nResult: ' + cleaner_results(self.result)
        else:
            self.desc += ''

        if self.consequence:
            self.desc += '\nConsequence: ' + cleaner_results(self.consequence)
        else:
            self.desc += ''

        self.ref = []
        self.cve = []
        for r in issue_node.findall('CVE_ID_LIST/CVE_ID'):
            self.node = r
            self.cve.append(self.get_text_from_subnode('ID'))
        for r in issue_node.findall('BUGTRAQ_ID_LIST/BUGTRAQ_ID'):
            self.node = r
            self.ref.append('bid-' + self.get_text_from_subnode('ID'))

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text
        return None


class QualysguardPlugin(PluginXMLFormat):
    """
    Example plugin to parse qualysguard output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["ASSET_DATA_REPORT", "SCAN"]
        self.id = 'Qualysguard'
        self.name = 'Qualysguard XML Output Plugin'
        self.plugin_version = '0.0.2'
        self.version = 'Qualysguard 8.17.1.0.2'
        self.framework_version = '1.0.0'
        self.options = None
        self.open_options = {"mode": "r", "encoding": "utf-8"}

    def parseOutputString(self, output):

        parser = QualysguardXmlParser(output)

        for item in parser.items:
            h_id = self.createAndAddHost(
                item.ip,
                item.os,
                hostnames=[item.hostname])

            for v in item.vulns:
                if v.port is None:
                    self.createAndAddVulnToHost(
                        h_id,
                        v.title if v.title else v.name,
                        ref=v.ref,
                        severity=v.severity,
                        resolution=v.solution if v.solution else '',
                        desc=v.desc,
                        external_id=v.external_id,
                        cve=v.cve,
                        cvss2=v.cvss2
                    )

                else:
                    web = False

                    try:
                        port = v.port
                        name = v.name
                    except (UnicodeDecodeError, AttributeError):
                        port = v.port
                        name = v.name

                    s_id = self.createAndAddServiceToHost(
                        h_id,
                        v.port,
                        v.protocol,
                        ports=[port],
                        status='open')
                    if port in ['80', '443'] or re.search('ssl|http', name):
                        web = True
                    else:
                        web = False

                    if web:
                        self.createAndAddVulnWebToService(
                            h_id,
                            s_id,
                            v.title if v.title else v.name,
                            ref=v.ref,
                            website=item.ip,
                            severity=v.severity,
                            desc=v.desc,
                            resolution=v.solution if v.solution else '',
                            external_id=v.external_id,
                            cve=v.cve,
                            cvss2=v.cvss2
                        )

                    else:
                        self.createAndAddVulnToService(
                            h_id,
                            s_id,
                            v.title if v.title else v.name,
                            ref=v.ref,
                            severity=v.severity,
                            desc=v.desc,
                            resolution=v.solution if v.solution else '',
                            external_id=v.external_id,
                            cve=v.cve,
                            cvss2=v.cvss2
                        )

        del parser


def createPlugin(*args, **kwargs):
    return QualysguardPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
import json
import shlex
import argparse
import urllib.parse as urlparse
from faraday_plugins.plugins.plugin import PluginBase
from faraday_plugins.plugins.plugins_utils import get_vulnweb_url_fields


__author__ = "Matías Lang"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Matías Lang"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Matías Lang"
__email__ = "matiasl@infobytesec.com"
__status__ = "Development"


status_codes = {
    200: "OK", 201:  "Created", 202:  "Accepted",
    203: "Non-Authoritative Information", 204:  "No Content",
    205: "Reset Content", 206:  "Partial Content", 207:  "Multi-Status",
    208: "Already Reported", 226:  "IM Used", 300:  "Multiple Choices",
    301: "Moved Permanently", 302:  "Found", 303:  "See Other",
    304: "Not Modified", 305:  "Use Proxy", 306:  "Switch Proxy",
    307: "Temporary Redirect", 308:  "Permanent Redirect",
    400: "Bad Request", 401:  "Unauthorized", 402:  "Payment Required",
    403: "Forbidden", 404:  "Not Found", 405:  "Method Not Allowed",
    406: "Not Acceptable", 407:  "Proxy Authentication Required",
    408: "Request Timeout", 409:  "Conflict", 410:  "Gone",
    411: "Length Required", 412:  "Precondition Failed",
    413: "Payload Too Large", 414:  "URI Too Long",
    415: "Unsupported Media Type", 416:  "Range Not Satisfiable",
    417: "Expectation Failed", 418:  "I'm a teapot",
    421: "Misdirected Request", 422:  "Unprocessable Entity", 423:  "Locked",
    424: "Failed Dependency", 426:  "Upgrade Required",
    428: "Precondition Required", 429:  "Too Many Requests",
    431: "Request Header Fields Too Large",
    451: "Unavailable For Legal Reasons", 500:  "Internal Server Error",
    501: "Not Implemented", 502:  "Bad Gateway", 503:  "Service Unavailable",
    504: "Gateway Timeout", 505:  "HTTP Version Not Supported",
    506: "Variant Also Negotiates", 507:  "Insufficient Storage",
    508: "Loop Detected", 510:  "Not Extended",
    511: "Network Authentication Required",
}


class DirsearchPlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "dirsearch"
        self.name = "dirsearch"
        self.plugin_version = "0.0.1"
        self.version = "0.0.1"
        self._command_regex = re.compile(r'^(sudo )?(python[0-9\.]? )?(dirsearch\.py)\s+?')
        self.addSetting("Ignore 403", str, "1")
        self._use_temp_file = True
        self._temp_file_extension = "json"

    def parseOutputString(self, output):
        self.parse_json(output)

    @property
    def should_ignore_403(self):
        val = self.getSetting('Ignore 403')
        if not val or not int(val):
            return False
        return True

    def parse_json(self, contents):
        try:
            data = json.loads(contents)
        except ValueError:
            self.logger.error('Error parsing report. Make sure the file has valid '
                     'JSON', 'ERROR')
            return
        for (base_url, items) in data.items():
            base_split = urlparse.urlsplit(base_url)
            ip = self.resolve_hostname(base_split.hostname)
            h_id = self.createAndAddHost(ip, hostnames=[base_split.hostname])
            s_id = self.createAndAddServiceToHost(
                h_id,
                base_split.scheme,
                'tcp',
                [base_split.port],
                status="open")

            for item in items:
                self.parse_found_url(base_url, h_id, s_id, item)

    def parse_found_url(self, base_url, h_id, s_id, item):
        if self.should_ignore_403 and item['status'] == 403:
            return
        url = urlparse.urlsplit(urlparse.urljoin(base_url, item['path']))
        response = "HTTP/1.1 {} {}\nContent-Length: {}".format(
            item['status'], status_codes.get(item['status'], 'unknown'),
            item['content-length'])
        redirect = item.get('redirect')
        if redirect is not None:
            response += f'\nLocation: {redirect}'
        self.createAndAddVulnWebToService(
            h_id,
            s_id,
            name=f'Path found: {item["path"]} ({item["status"]})',
            desc=f"Dirsearch tool found the following URL: {url.geturl()}",
            severity="info",
            method='GET',
            response=response,
            **get_vulnweb_url_fields(url.geturl()))

    def processCommandString(self, username, current_path, command_string):
        parser = argparse.ArgumentParser(conflict_handler='resolve')
        parser.add_argument('-h', '--help', action='store_true')
        parser.add_argument('--json-report')
        args, unknown = parser.parse_known_args(shlex.split(command_string))
        if args.help:
            return None
        if args.json_report:
            # The user already defined a path to the JSON report
            self._output_file_path = args.json_report
            return None
        else:
            super().processCommandString(username, current_path, command_string)
            return f'{command_string} --json-report {self._output_file_path}'


def createPlugin(*args, **kwargs):
    return DirsearchPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from urllib.parse import urlsplit
import ipaddress

from lxml import etree

from faraday_plugins.plugins.plugin import PluginJsonFormat
from faraday_plugins.plugins.repo.acunetix.DTO import Acunetix, Scan
from json import loads

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"

from faraday_plugins.plugins.repo.acunetix_json.DTO import AcunetixJsonParser, Vulnerabilities, \
    VulnerabilityTypes
from faraday_plugins.plugins.plugins_utils import its_cwe


class AcunetixXmlParser:
    """
    The objective of this class is to parse an xml file generated by
    the acunetix tool.

    TODO: Handle errors.
    TODO: Test acunetix output version. Handle what happens if
    the parser doesn't support it.
    TODO: Test cases.

    @param acunetix_xml_filepath A proper xml generated by acunetix
    """

    def __init__(self, xml_output):

        tree = self.parse_xml(xml_output)
        self.acunetix = Acunetix(tree)

    @staticmethod
    def parse_xml(xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead
        of reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """

        try:
            parser = etree.XMLParser(recover=True)
            tree = etree.fromstring(xml_output, parser=parser)
        except SyntaxError as err:
            print("SyntaxError: %s. %s", err, xml_output)
            return None

        return tree


class AcunetixJsonPlugin(PluginJsonFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Acunetix_Json"
        self.name = "Acunetix JSON Output Plugin"
        self.plugin_version = "0.1"
        self.version = "9"
        self.json_keys = {'export'}
        self.framework_version = "1.0.0"
        self._temp_file_extension = "json"
    """
    Example plugin to parse acunetix output.
    """

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """
        parser = AcunetixJsonParser(loads(output))
        for site in parser.export.scans:
            self.new_structure(site)

    def new_structure(self, site: Scan):
        start_url = site.info.host
        if site.info.start_url:
            start_url = site.info.start_url
        url_data = urlsplit(start_url)
        site_ip = self.resolve_hostname(url_data.hostname)
        ports = '443' if (url_data.scheme == 'https') else '80'
        vulnerability_type = {i.vt_id: i for i in site.vul_types}
        h_id = self.createAndAddHost(site_ip, None, hostnames=[url_data.hostname])
        s_id = self.createAndAddServiceToHost(
            h_id,
            "http",
            "tcp",
            ports=[ports],
            version=None,
            status='open')
        for i in site.vulnerabilities:
            vul_type = vulnerability_type[i.info.vt_id]
            cwe = its_cwe(vul_type.tags)
            self.create_vul(i, vul_type, h_id, s_id, url_data, cwe)

    def create_vul(self, vul: Vulnerabilities, vul_type: VulnerabilityTypes, h_id, s_id, url_data, cwe):
        cvss3 = {
            'vector_string': vul_type.cvss3_vector
        }
        cvss2 = {
            'vector_string': vul_type.cvss2_vector
        }
        self.createAndAddVulnWebToService(
            h_id,
            s_id,
            vul_type.name,
            vul_type.description,
            website=url_data.hostname,
            severity=vul_type.severity,
            resolution=vul_type.recommendation,
            request=vul.info.request,
            response=vul.response,
            cwe=cwe,
            cvss3=cvss3,
            cvss2=cvss2
        )

    @staticmethod
    def get_domain(scan: Scan):
        url = scan.start_url
        if not url.startswith('http'):
            url = f'http://{url}'
        url_data = urlsplit(url)
        if not url_data.scheme:
            url_data = urlsplit(scan.crawler.start_url_attr)
        return url_data


def createPlugin(*args, **kwargs):
    return AcunetixJsonPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2016  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = 'Blas Moyano'
__copyright__ = 'Copyright 2020, Faraday Project'
__credits__ = ['Blas Moyano']
__license__ = ''
__version__ = '1.0.0'
__status__ = 'Development'


class CheckmarxXmlParser:
    def __init__(self, xml_output):
        self.tree = self.parse_xml(xml_output)
        if self.tree:
            self.cx_xml_results_attribs = self.tree.attrib
            self.query = self.getQuery(self.tree)
        else:
            self.query = None

    def parse_xml(self, xml_output):
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f'SyntaxError In xml: {err}. {xml_output}')
            return None
        return tree

    def getQuery(self, tree):
        for self.query_node in tree:
            yield Querys(self.query_node)


class Querys():
    def __init__(self, query_node):

        self.query_attrib = query_node.attrib
        self.result, self.path, self.path_node = self.get_Result(query_node.findall('Result'))

    def get_Result(self, result):
        result_atr = []
        path = []
        path_node = []
        for r in result:
            self.result_attrib = r.attrib
            self.path_attrib = r.find('Path').attrib
            self.path_node = self.get_path_node_info(r.find('Path').findall('PathNode'))
            result_atr.append(self.result_attrib)
            path.append(self.path_attrib)
            path_node.append(self.path_node)
        return result_atr, path, path_node

    def get_path_node_info(self, path_node):
        lista = []
        for pn in path_node:
            lista_v = []
            for info_pn in pn:
                if info_pn.tag == 'Snippet':
                    valor = (
                        'Number', info_pn.find('Line').find('Number').text, 'Code',
                        info_pn.find('Line').find('Code').text)
                else:
                    valor = (info_pn.tag, info_pn.text)
                lista_v.append(valor)
            lista.append(lista_v)
        return lista


class CheckmarxPlugin(PluginXMLFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["CxXMLResults"]
        self.id = 'Checkmarx'
        self.name = 'Checkmarx XML Output Plugin'
        self.plugin_version = '1.0.0'
        self.version = '1.0.0'
        self.framework_version = '1.0.0'
        self.options = None

    def parseOutputString(self, output):
        parser = CheckmarxXmlParser(output)
        if not parser.query:
            self.logger.warning('Error in xml report... Exiting...')
            return

        url = urlparse(parser.cx_xml_results_attribs['DeepLink'])
        port = url.port
        if not port:
            if url.scheme == 'https':
                port = 443
            elif url.scheme == 'http':
                port = 80
            else:
                port = 0
        project_name = 'ProjectName' in parser.cx_xml_results_attribs
        host_id = self.createAndAddHost(url.hostname, hostnames=[url.hostname, url.netloc])
        service_to_interface = self.createAndAddServiceToHost(host_id, name=url.scheme, ports=port)
        for vulns in parser.query:
            refs = []
            categories = 'categories' in vulns.query_attrib
            vuln_desc = ''
            if categories:
                vuln_desc = vulns.query_attrib['categories']
            vuln_name = vulns.query_attrib['name']
            vuln_severity = vulns.query_attrib['Severity']
            vuln_external_id = vulns.query_attrib['id']
            cwe = [f'CWE-{vulns.query_attrib["cweId"]}']
            data = ''
            for files_data in vulns.path_node:
                for file_data in files_data:
                    data += 60 * '-' + '\n'
                    for row_data in file_data:
                        data += ' '.join([data for data in row_data if data]) + '\n'

            for v_result in vulns.result:
                refs.append(v_result['DeepLink'])
                refs.append(v_result['FileName'])

            self.createAndAddVulnToHost(host_id, vuln_name, severity=vuln_severity,
                                        resolution=data, external_id=vuln_external_id, cwe=cwe)

            self.createAndAddVulnWebToService(host_id, service_to_interface, vuln_name,
                                              desc=vuln_desc, severity=vuln_severity,
                                              resolution=data, ref=refs, cwe=cwe)


def createPlugin(*args, **kwargs):
    return CheckmarxPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from faraday_plugins.plugins.plugin import PluginBase
import re



__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class TheharvesterParser:
    """
    The objective of this class is to parse an xml file generated by the theharvester tool.

    TODO: Handle errors.
    TODO: Test theharvester output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param theharvester_filepath A proper simple report generated by theharvester
    """

    def __init__(self, output):

        self.items = []
        _hosts, _vhosts = [], []

        mregex = re.search(
            r"\[\+\] Hosts found in search engines:[-=\s]+([\w\W]*)\[\+\]", output)
        if mregex is None:
            mregex = re.search(
                "\\[\\+\\] Hosts found in search engines:[-=\\s]+([\\w\\W]*)\n", output)
        mregex2 = re.search("\\[\\+\\] Virtual hosts:[-=\\s]+([\\w\\W]*)\n", output)

        if mregex is None and mregex2 is None:
            return

        if mregex:
            _hosts = mregex.group(1).strip().split("\n")
        if mregex2:
            _vhosts = mregex2.group(1).strip().split("\n")

        for line in _hosts:

            info = line.split(":")

            if len(info) > 1:
                item = {'host': info[1].strip(), 'ip': info[0].strip()}

                self.items.append(item)

        for line in _vhosts:

            info = line.split()
            if len(info) > 1:
                item = {'host': info[1].strip(), 'ip': info[0].strip()}

                self.items.append(item)


class TheharvesterPlugin(PluginBase):
    """
    Example plugin to parse theharvester output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Theharvester"
        self.name = "Theharvester XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "2.2a"
        self.options = None
        self._current_output = None
        self._current_path = None
        self._command_regex = re.compile(
            r'^(sudo theHarvester\.py|theHarvester\.py|python theHarvester\.py|\.\/theHarvester\.py)\s+.*?')
        self._completition = {
            "": "Examples:./theharvester.py -d microsoft.com -l 500 -b google",
            "-d": "Domain to search or company name",
            "-b": "Data source (google,bing,bingapi,pgp,linkedin,google-profiles,exalead,all)",
            "-s": "Start in result number X (default 0)",
            "-v": "Verify host name via dns resolution and search for vhosts(basic)",
            "-l": "Limit the number of results to work with(bing goes from 50 to 50 results,",
            "-f": "Save the results into an XML file",
            "-n": "Perform a DNS reverse query on all ranges discovered",
            "-c": "Perform a DNS brute force for the domain name",
            "-t": "Perform a DNS TLD expansion discovery",
            "-e": "Use this DNS server",
            "-h": "use SHODAN database to query discovered hosts. google 100 to 100, and pgp doesn't use this option)",
        }


    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.
        """
        parser = TheharvesterParser(output)
        for item in parser.items:
            host = []
            if item['host'] != item['ip']:
                host = [item['host']]
            h_id = self.createAndAddHost(item['ip'], hostnames=host)
        del parser



def createPlugin(*args, **kwargs):
    return TheharvesterPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from re import findall
from urllib.parse import urlsplit

from lxml import etree

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.repo.acunetix.DTO import Acunetix, Scan

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class AcunetixXmlParser:
    """
    The objective of this class is to parse an xml file generated by
    the acunetix tool.

    TODO: Handle errors.
    TODO: Test acunetix output version. Handle what happens if
    the parser doesn't support it.
    TODO: Test cases.

    @param acunetix_xml_filepath A proper xml generated by acunetix
    """

    def __init__(self, xml_output):

        tree = self.parse_xml(xml_output)
        self.acunetix = Acunetix(tree)

    @staticmethod
    def parse_xml(xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead
        of reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """

        try:
            parser = etree.XMLParser(recover=True)
            tree = etree.fromstring(xml_output, parser=parser)
        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree


class AcunetixPlugin(PluginXMLFormat):
    """
    Example plugin to parse acunetix output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "ScanGroup"
        self.id = "Acunetix"
        self.name = "Acunetix XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "9"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self.target = None

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """
        parser = AcunetixXmlParser(output)

        for site in parser.acunetix.scan:
            url_data = self.get_domain(site)
            if not url_data:
                continue
            if url_data.hostname:
                self.old_structure(url_data, site)
            else:
                self.new_structure(site)

    def new_structure(self, site):
        for item in site.reportitems.reportitem:
            if not item.technicaldetails.request:
                self.logger.warning("No request data")
                continue
            request_host = findall('Host: (.*)', item.technicaldetails.request)
            if request_host:
                host = request_host[0]
                url = f'http://{host}'
                url_data = urlsplit(url)
                site_ip = self.resolve_hostname(host)
                h_id = self.createAndAddHost(site_ip, site.os, hostnames=[host])
                s_id = self.createAndAddServiceToHost(
                    h_id,
                    "http",
                    "tcp",
                    ports=['443'],
                    version=site.banner,
                    status='open')
                self.create_vul(item, h_id, s_id, url_data)
            else:
                self.logger.warning("No host in request")

    def old_structure(self, url_data, site: Scan):
        site_ip = self.resolve_hostname(url_data.hostname)
        if url_data.hostname:
            hostnames = [url_data.hostname]
        else:
            hostnames = None
        port = url_data.port or (443 if url_data.scheme == "https" else 80)

        h_id = self.createAndAddHost(site_ip, site.os, hostnames=hostnames)
        s_id = self.createAndAddServiceToHost(
            h_id,
            "http",
            "tcp",
            ports=[port],
            version=site.banner,
            status='open')
        for item in site.reportitems.reportitem:
            self.create_vul(item, h_id, s_id, url_data)

    def create_vul(self, item, h_id, s_id, url_data):
        description = item.description
        cvss3 = {}
        if item.cvss3.node is not None:
            cvss3['vector_string'] = item.cvss3.descriptor
        cvss2 = {}
        if item.cvss.node is not None:
            cvss2['vector_string'] = item.cvss.descriptor
        if item.affects:
            description += f'\nPath: {item.affects}'
        if item.parameter:
            description += f'\nParameter: {item.parameter}'
        try:
            cve = [item.cvelist.cve.text if item.cvelist.cve else ""]
        except Exception:
            cve = []
        try:
            cwe = [item.cwelist.cwe.text if item.cwelist.cwe else ""]
        except:
            cwe = []
        self.createAndAddVulnWebToService(
            h_id,
            s_id,
            item.name,
            description,
            website=url_data.hostname,
            severity=item.severity,
            resolution=item.recommendation,
            path=item.affects,
            params=item.parameter,
            request=item.technicaldetails.request,
            response=item.technicaldetails.response,
            ref=[i.url for i in item.references.reference],
            cve=cve,
            cwe=cwe,
            cvss2=cvss2,
            cvss3=cvss3)

    @staticmethod
    def get_domain(scan: Scan):
        url = scan.start_url
        if not url.startswith('http'):
            url = f'http://{url}'
        url_data = urlsplit(url)
        if not url_data.scheme:
            url_data = urlsplit(scan.crawler.start_url_attr)
        return url_data


def createPlugin(*args, **kwargs):
    return AcunetixPlugin(*args, **kwargs)



import getpass
import io
import json
import logging
import os
import shlex
import subprocess
import sys
from pathlib import Path

import click
from tabulate import tabulate

from faraday_plugins import __version__
from faraday_plugins.plugins.manager import PluginsManager, ReportAnalyzer, CommandAnalyzer
from faraday_plugins.plugins.plugin import PluginByExtension

root_logger = logging.getLogger("faraday")
if not root_logger.handlers:
    PLUGIN_DEBUG = os.environ.get("PLUGIN_DEBUG", "0")
    if PLUGIN_DEBUG == "1":
        out_hdlr = logging.StreamHandler(sys.stdout)
        out_hdlr.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s [%(filename)s:%(lineno)s - %(funcName)s()]  %(message)s'))
        out_hdlr.setLevel(logging.DEBUG)
        root_logger.addHandler(out_hdlr)
        root_logger.setLevel(logging.DEBUG)

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__, '-v', '--version')
def cli():
    pass


@cli.command()
@click.option('-cpf', '--custom-plugins-folder', type=str)
def list_plugins(custom_plugins_folder):
    plugins_manager = PluginsManager(custom_plugins_folder)
    click.echo(click.style(f"Faraday Plugins v{__version__}", fg="cyan"))
    click.echo(click.style("Available Plugins :", fg="cyan"))
    loaded_plugins = 0
    plugins_data = []
    for plugin_id, plugin in plugins_manager.get_plugins():
        console_enabled = plugin._command_regex is not None
        console_enabled_color = "green" if console_enabled else "red"
        console_enabled_text = click.style(f"{'Yes' if console_enabled else 'No'}", fg=console_enabled_color)
        report_enabled = isinstance(plugin, PluginByExtension)
        report_enabled_color = "green" if report_enabled else "red"
        report_enabled_text = click.style(f"{'Yes' if report_enabled else 'No'}", fg=report_enabled_color)
        plugins_data.append({"Name": plugin.name, "ID": plugin.id, "Command": console_enabled_text,
                             "Report": report_enabled_text})
    click.echo(tabulate(
        plugins_data,
        headers="keys",
        tablefmt="github",
    ))
    click.echo(click.style(f"Loaded Plugins: {len(plugins_data)}", fg="cyan"))


@cli.command()
@click.argument('report_file')
@click.option('--plugin_id', type=str)
@click.option('-cpf', '--custom-plugins-folder', type=str)
@click.option('--summary', is_flag=True)
@click.option('-o', '--output-file', type=click.Path(exists=False))
@click.option('--ignore-info', is_flag=True, help="Ignore information vulnerabilities")
@click.option('-drh', '--dont-resolve-hostname', is_flag=True, help="Dont resolve hostname", default=False)
@click.option('--vuln-tag', help="Vuln tag", default=None)
@click.option('--service-tag', help="Service tag", default=None)
@click.option('--host-tag', help="Host tag", default=None)
def process_report(report_file, plugin_id, custom_plugins_folder, summary, output_file, ignore_info,
                   dont_resolve_hostname, vuln_tag, service_tag, host_tag):
    if not os.path.isfile(report_file):
        click.echo(click.style(f"File {report_file} Don't Exists", fg="red"), err=True)
    else:
        plugins_manager = PluginsManager(custom_plugins_folder,
                                         ignore_info=ignore_info,
                                         hostname_resolution=not dont_resolve_hostname,
                                         vuln_tag=vuln_tag,
                                         service_tag=service_tag,
                                         host_tag=host_tag)
        analyzer = ReportAnalyzer(plugins_manager)
        if plugin_id:
            plugin = plugins_manager.get_plugin(plugin_id)
            if not plugin:
                click.echo(click.style(f"Invalid Plugin: {plugin_id}", fg="red"), err=True)
                return
        else:
            plugin = analyzer.get_plugin(report_file)
            if not plugin:
                click.echo(click.style(f"Failed to detect report: {report_file}", fg="red"), err=True)
                return
        plugin.processReport(Path(report_file), getpass.getuser())
        if summary:
            click.echo(json.dumps(plugin.get_summary(), indent=4))
        else:
            if output_file:
                with open(output_file, "w") as f:
                    json.dump(plugin.get_data(), f)
            else:
                click.echo(json.dumps(plugin.get_data(), indent=4))


@cli.command()
@click.argument('command')
@click.option('--plugin_id', type=str)
@click.option('-cpf', '--custom-plugins-folder', type=str)
@click.option('-dr', '--dont-run', is_flag=True)
@click.option('--summary', is_flag=True)
@click.option('-o', '--output-file', type=click.Path(exists=False))
@click.option('-sh', '--show-output', is_flag=True)
@click.option('--ignore-info', is_flag=True, help="Ignore information vulnerabilities")
@click.option('--hostname-resolution', is_flag=True, help="Resolve hostname")
@click.option('--vuln-tag', help="Vuln tag", default=None)
@click.option('--service-tag', help="Service tag", default=None)
@click.option('--host-tag', help="Host tag", default=None)
def process_command(command, plugin_id, custom_plugins_folder, dont_run, summary, output_file, show_output,
                    ignore_info, hostname_resolution, vuln_tag, service_tag, host_tag):
    plugins_manager = PluginsManager(custom_plugins_folder,
                                     ignore_info=ignore_info,
                                     hostname_resolution=hostname_resolution,
                                     vuln_tag=vuln_tag,
                                     service_tag=service_tag,
                                     host_tag=host_tag)
    analyzer = CommandAnalyzer(plugins_manager)
    if plugin_id:
        plugin = plugins_manager.get_plugin(plugin_id)
        if not plugin:
            click.echo(click.style(f"Invalid Plugin: {plugin_id}", fg="red"), err=True)
            return
    else:
        plugin = analyzer.get_plugin(command)
        if not plugin:
            click.echo(click.style(f"Failed to detect command: {command}", fg="red"), err=True)
            return
    current_path = os.path.abspath(os.getcwd())
    modified_command = plugin.processCommandString(getpass.getuser(), current_path, command)
    if modified_command:
        command = modified_command
    if dont_run:
        color_message = click.style("Command: ", fg="green")
        click.echo(f"{color_message} {command}")
    else:
        p = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = io.StringIO()
        while True:
            retcode = p.poll()
            line = p.stdout.readline().decode('utf-8')
            if show_output:
                sys.stdout.write(line)
            output.write(line)
            if retcode is not None:
                extra_lines = map(lambda x: x.decode('utf-8'), p.stdout.readlines())
                if show_output:
                    sys.stdout.writelines(line)
                output.writelines(extra_lines)
                break
        output_value = output.getvalue()
        if retcode == 0:
            plugin.processOutput(output_value)
            if summary:
                click.echo(json.dumps(plugin.get_summary(), indent=4))
            else:
                if output_file:
                    with open(output_file, "w") as f:
                        json.dump(plugin.get_data(), f)
                else:
                    click.echo(json.dumps(plugin.get_data(), indent=4))
        else:
            click.echo(click.style("Command execution error!!", fg="red"), err=True)



@cli.command()
@click.argument('report_file')
@click.option('-cpf', '--custom-plugins-folder', type=str)
def detect_report(report_file, custom_plugins_folder):
    if not os.path.isfile(report_file):
        click.echo(click.style(f"File {report_file} Don't Exists", fg="red"))
    else:
        plugins_manager = PluginsManager(custom_plugins_folder)
        analyzer = ReportAnalyzer(plugins_manager)
        plugin = analyzer.get_plugin(report_file)
        if plugin:
            click.echo(click.style(f"Faraday Plugin: {plugin.id}", fg="cyan"))
        else:
            click.echo(click.style(f"Failed to detect report: {report_file}", fg="red"), err=True)


@cli.command()
@click.argument('command')
@click.option('-cpf', '--custom-plugins-folder', type=str)
def detect_command(command, custom_plugins_folder):
    plugins_manager = PluginsManager(custom_plugins_folder)
    analyzer = CommandAnalyzer(plugins_manager)
    plugin = analyzer.get_plugin(command)
    if plugin:
        click.echo(click.style(f"Faraday Plugin: {plugin.id}", fg="cyan"))
    else:
        click.echo(click.style(f"Failed to detect command: {command}", fg="red"), err=True)
